@@ -26,22 +26,26 @@
 ;;
 
 (defvar mew-addrbook-orig-alist nil
-  "(key addr) or (key (addr1, addr2) nickname name)")
+  "(key (addr1, addr2) nickname name) or (key addr)")
 
 (defsubst mew-adbkorigent-by-shortname (user)
   (mew-addrbook-alias-hit user mew-addrbook-orig-alist))
 
 ;;
 
-(defvar mew-addrbook-alist nil
+(defvar mew-addrbook-alist nil ;; mew-alias-auto-alist is appended
   "(key addr) or (key (addr1, addr2) nickname name)")
 
 (defsubst mew-adbkent-by-addr-with-alist (addr alist)
-  (mew-assoc-member-case-equal addr mew-addrbook-alist 1))
+  (mew-assoc-member-case-equal addr alist 1))
 
 (defsubst mew-adbkent-by-addr (addr)
   (mew-adbkent-by-addr-with-alist addr mew-addrbook-alist))
 
+(defsubst mew-adbkent-by-addr2 (addr)
+  ;; for nickname and name, mew-addrbook-orig-alist is enough and fast
+  (mew-adbkent-by-addr-with-alist addr mew-addrbook-orig-alist))
+
 (defsubst mew-adbkent-shortname (adbkent)
   (nth 0 adbkent))
 
@@ -61,10 +65,10 @@
   (mew-adbkent-addrs (mew-adbkent-by-addr addr)))
 
 (defun mew-addrbook-nickname-get (addr)
-  (mew-adbkent-nickname (mew-adbkent-by-addr addr)))
+  (mew-adbkent-nickname (mew-adbkent-by-addr2 addr)))
 
 (defun mew-addrbook-name-get (addr)
-  (mew-adbkent-name (mew-adbkent-by-addr addr)))
+  (mew-adbkent-name (mew-adbkent-by-addr2 addr)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -238,14 +242,14 @@ returned."
       (setq pars (cdr pars))
       (when (file-readable-p par)
 	(setq ret t)
-	(insert-file-contents par)
+	(mew-insert-file-contents par)
 	(setq path (file-name-directory par))
 	;; include children files
 	(while (re-search-forward "^<[ \t]*\\([^ \t\n]+\\).*$" nil t)
 	  (setq chr (expand-file-name (mew-match-string 1) path))
 	  (replace-match "" nil t)
 	  (when (and (file-readable-p chr) (not (member chr files)))
-	    (insert-file-contents chr)
+	    (mew-insert-file-contents chr)
 	    (setq files (cons chr files))))
 	(goto-char (point-max))))
     ;; remove commets
@@ -451,7 +455,7 @@ The keys that are defined for this mode are:
     (if errmsg
 	(message "%s" errmsg)
       (save-excursion
-	(setq buf (find-file-noselect
+	(setq buf (mew-find-file-noselect2
 		   (expand-file-name mew-addrbook-file mew-conf-path)))
 	(set-buffer buf)
 	(goto-char (point-min))
@@ -505,7 +509,7 @@ The keys that are defined for this mode are:
 ;;; Editing auto-alist
 ;;;
 
-(defun mew-summary-addrbook-read-buffer ()
+(defun mew-summary-alias-read-buffer ()
   (save-excursion
     (goto-char (point-min))
     ;; An error occurs if the expression is broken.
@@ -514,7 +518,11 @@ The keys that are defined for this mode are:
     (mew-addrbook-concat-uniq)
     nil)) ;; to write
 
-(defun mew-summary-addrbook-edit ()
+(defun mew-summary-alias-edit ()
+  "Editing the auto alias file which contains a list of alias-address
+pairs. Remove unnecessary entries and save the file by
+'\\[save-buffer]'. After saving, the modification is automatically
+reflected."
   (interactive)
   (let ((file mew-alias-auto-file))
     (setq mew-alias-auto-alist
@@ -522,9 +530,11 @@ The keys that are defined for this mode are:
     (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist)
     (unless (file-name-absolute-p file)
       (setq file (expand-file-name file mew-conf-path)))
-    (find-file file)
+    (switch-to-buffer (mew-find-file-noselect file))
     (emacs-lisp-mode)
-    (add-hook 'local-write-file-hooks 'mew-summary-addrbook-read-buffer)))
+    (if (boundp 'write-file-functions)
+	(add-hook 'write-file-functions 'mew-summary-alias-read-buffer nil 'local)
+      (add-hook 'local-write-file-hooks 'mew-summary-alias-read-buffer))))
 
 (provide 'mew-addrbook)
 
