@@ -73,7 +73,7 @@ Please send comments to Kazu@Mew.org.
 				   (expand-file-name mew-icon-mew
 						     mew-icon-directory)))))
 	(indent-to (startup-center-spaces mew-logo))
-	(set-extent-begin-glyph (mew-overlay-make (point) (point)) mew-logo)
+	(set-extent-end-glyph (mew-overlay-make (point) (point)) mew-logo)
 	(goto-char (point-min))
 	(mew-redraw time))
        ((and mew-gemacs-p window-system
@@ -82,8 +82,9 @@ Please send comments to Kazu@Mew.org.
 	(goto-char (point-max))
 	(setq mew-logo (create-image 
 			(expand-file-name mew-icon-mew mew-icon-directory)))
-	(setq indent (/ (- (frame-width)
+	(setq indent (/ (- (window-width)
 			   (truncate (car (image-size mew-logo)))) 2))
+	(setq indent (max 0 indent))
 	(insert (make-string indent 32))
 	(insert-image mew-logo)
 	(goto-char (point-min))
@@ -91,7 +92,7 @@ Please send comments to Kazu@Mew.org.
        ((and (featurep 'bitmap) window-system)
 	(or
 	 (condition-case nil
-	     (insert-file-contents
+	     (mew-insert-file-contents
 	      (expand-file-name
 	       mew-icon-mew-mule-bitmap-image mew-icon-directory))
 	   (file-error nil))
