@@ -44,7 +44,9 @@
   (setq comment-start ">")
   (make-local-hook 'after-change-functions)
   (add-hook 'after-change-functions 'mew-draft-dynamic-highlight nil 'local)
-  (add-hook 'local-write-file-hooks 'mew-encode-make-backup)
+  (if (boundp 'write-file-functions)
+      (add-hook 'write-file-functions 'mew-encode-make-backup nil 'local)
+    (add-hook 'local-write-file-hooks 'mew-encode-make-backup))
   (make-local-variable 'after-save-hook)
   (when mew-require-final-newline
     (make-local-variable 'require-final-newline)
@@ -262,7 +264,7 @@ Moreover, '~/.mew-fib' supports aliases like;
       (setq switch-func 'switch-to-buffer-other-window)))
     (mew-frwlet
      mew-cs-m17n mew-cs-dummy
-     (funcall switch-func (find-file-noselect draft-path)))
+     (funcall switch-func (mew-find-file-noselect draft-path)))
     ;; draft buffer
     (mew-set-buffer-cs mew-cs-m17n)
     ;; copy config, first
@@ -382,7 +384,7 @@ the Body: field."
 	     (file-exists-p (expand-file-name mew-x-face-file)))
 	(let (xface)
 	  (with-temp-buffer
-	    (insert-file-contents (expand-file-name mew-x-face-file))
+	    (mew-insert-file-contents (expand-file-name mew-x-face-file))
 	    (setq xface (mew-buffer-substring (point-min)
 					      (max (buffer-size) 1))))
 	  (mew-draft-header-insert mew-x-face: xface)))
@@ -442,11 +444,15 @@ citation prefix and label.
 	(set-buffer (mew-buffer-message))
 	(save-restriction
 	  (widen)
-	  (let ((mark-active t) (transient-mark-mode nil))
+	  (let ((mark-active t))
 	    (cond
 	     (arg 
 	      (setq beg (point-min) end (point-max)))
-	     ((mew-mark) 
+	     ((and (not mew-cite-ignore-region)
+		   (mew-mark)
+		   (/= (point) (mew-mark))
+		   (not (and mew-cite-ignore-mouse-region
+			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
 	     ((mew-msghder-p)
 	      ;; header exists in Message mode
@@ -455,7 +461,7 @@ citation prefix and label.
 	     (t
 	      (setq beg (point-min) end (point-max)))))
 	  (setq cite (mew-buffer-substring beg end))))
-      (push-mark (point) t t) ;; for C-x C-x
+      (mew-push-mark)
       (insert cite)
       (mew-draft-auto-set-input-method))))
 
@@ -498,11 +504,14 @@ citation prefix and label.
 	(save-restriction
 	  ;; first prepare "cite"
 	  (widen)
-	  (let ((mark-active t) (transient-mark-mode nil))
+	  (let ((mark-active t))
 	    (cond
 	     ;; arg will be effect in mew-cite-original
-	     ((and (mew-mark) (or (not mew-cite-ignore-mouse-region)
-				  (not (mew-mouse-region-p))))
+	     ((and (not mew-cite-ignore-region)
+		   (mew-mark)
+		   (/= (point) (mew-mark))
+		   (not (and mew-cite-ignore-mouse-region
+			     (mew-mouse-region-p))))
 	      (setq beg (region-beginning) end (region-end)))
 	     ((mew-msghder-p)
 	      ;; header exists in Message mode. Skip the header
@@ -539,7 +548,7 @@ citation prefix and label.
           (save-excursion
             (let* ((mew-references-max-count nil)
 		   (irt (mew-header-get-value mew-in-reply-to:))
-		   (irtl (mew-idstr-to-id-list irt nil 'rev))
+		   (irtl (mew-idstr-to-id-list irt 'rev))
 		   irtstr)
 	      (mew-addq irtl irt-msgid)
 	      (setq irtl (nreverse irtl))
@@ -551,6 +560,7 @@ citation prefix and label.
 	;; this gets complicated due to supercite, please do not care
 	(narrow-to-region (point)(point)) ;; for (goto-char (point-min))
 	(insert cite)
+	;; not for C-x C-x. Do not use mew-push-mark.
 	(push-mark (point) t t)
 	(goto-char (point-min)))
       (cond
@@ -585,7 +595,7 @@ citation prefix and label.
 	  ;; header has been already cited. So, delete it.
 	  (delete-region beg (progn (mew-header-goto-body) (point))))
       (insert label)
-      (push-mark (point) t t) ;; for C-x C-x
+      (mew-push-mark)
       (and (bolp) (insert prefix))
       (while (= 0 (forward-line))
 	(or (= (point) (point-max))
@@ -660,33 +670,35 @@ The citation style is 'from_address> ', e.g. 'kazu> '"
 
 (defun mew-draft-save-buffer ()
   (interactive)
-  (save-excursion
-    (mew-header-clear)
-    (insert-before-markers "\n") ;; for mew-summary-reply
-    (save-buffer)
-    (delete-region (1- (point)) (point))
-    (mew-header-prepared)
-    (set-buffer-modified-p nil)))
+  (let ((after-change-functions nil))
+    (save-excursion
+      (mew-header-clear 'keep-read-only)
+      (insert-before-markers "\n") ;; for mew-summary-reply
+      (save-buffer)
+      (delete-region (1- (point)) (point))
+      (mew-header-prepared)
+      (set-buffer-modified-p nil))))
 
 (defun mew-draft-kill ()
   "Kill this draft."
   (interactive)
-  (if (y-or-n-p "Kill draft message? ")
-      (let* ((attachdir (mew-attachdir)) ;; attachdir must be here
-	     (draft (buffer-file-name))
-	     (buf (current-buffer))
-	     (mdi (concat draft mew-draft-info-suffix)))
-	(mew-overlay-delete-buffer)
-	(save-buffer)
-	(mew-delete-file draft)
-	(mew-delete-file mdi)
-	(if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
-	    (delete-frame)
-	  (mew-current-get-window-config))
-	(mew-remove-buffer buf)
-	(mew-delete-directory-recursively attachdir)
-	(message "Draft was killed"))
-    (message "Draft was not killed")))
+  (if (not (y-or-n-p "Kill draft message? "))
+      (message "Draft was not killed")
+    (let* ((attachdir (mew-attachdir)) ;; attachdir must be here
+	   (draft (buffer-file-name))
+	   (buf (current-buffer))
+	   (mdi (concat draft mew-draft-info-suffix)))
+      (mew-elet
+       (mew-overlay-delete-buffer))
+      (save-buffer)
+      (mew-delete-file draft)
+      (mew-delete-file mdi)
+      (if (and (mew-tinfo-get-other-frame) (> (length (frame-list)) 1))
+	  (delete-frame)
+	(mew-current-get-window-config))
+      (mew-delete-directory-recursively attachdir)
+      (mew-remove-buffer buf)
+      (message "Draft was killed"))))
 
 (defun mew-draft-insert-signature (&optional arg)
   "Insert the signature file specified by mew-signature-file.
@@ -725,7 +737,7 @@ you can set the case."
 	    (goto-char (1- (mew-attach-begin))))
 	  (end-of-line)
 	  (unless (bolp) (insert "\n")))
-	(insert-file-contents sigfile)))))
+	(mew-insert-file-contents sigfile)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -829,10 +841,10 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
   (run-hooks 'mew-send-hook)
   (let* ((buf (current-buffer))
 	 (pnm (mew-smtp-info-name case))
-	 resentp fcc sendit err)
+	 resentp fcc sendit msg err)
     (if (get-process pnm)
 	(message "Another message is being sent. Try later")
-      (mew-draft-remove-illegal-fields)
+      (mew-draft-remove-invalid-fields)
       ;; Check resentp
       (save-excursion
 	(goto-char (point-min))
@@ -857,11 +869,10 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (if (mew-smtp-encode pnm case resentp fcc privacy signer)
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
-		  (cond
-		   ((eq action 'queue)
-		    (mew-smtp-queue case "from Draft mode"))
-		   ((eq action 'send)
-		    (mew-smtp-send-message case))))
+		  (setq msg (mew-smtp-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-smtp-send-message 
+		       case (mew-queue-folder case) (list msg))))
 	      (setq err t))))
 	(if (not err)
 	    (run-hooks 'mew-real-send-hook)
@@ -873,10 +884,10 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
   (run-hooks 'mew-post-hook)
   (let* ((buf (current-buffer))
 	 (pnm (mew-nntp2-info-name case))
-	 fcc sendit err)
+	 fcc sendit msg err)
     (if (get-process pnm)
 	(message "Another message is being posted. Try later")
-      (mew-draft-remove-illegal-fields)
+      (mew-draft-remove-invalid-fields)
       ;; Ask Subject: before the query of "Really post".
       ;; Typing C-g here gets back to the draft.
       (mew-encode-ask-subject)
@@ -897,11 +908,10 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	    (if (mew-nntp2-encode pnm case fcc privacy signer)
 		(let ((mdi (concat (buffer-file-name) mew-draft-info-suffix)))
 		  (mew-delete-file mdi)
-		  (cond
-		   ((eq action 'queue)
-		    (mew-nntp2-queue case "from Draft mode"))
-		   ((eq action 'send)
-		    (mew-nntp2-send-message case))))
+		  (setq msg (mew-nntp2-queue case "from Draft mode"))
+		  (if (eq action 'send)
+		      (mew-nntp2-send-message 
+		       case (mew-postq-folder case) (list msg))))
 	      (setq err t))))
 	(if (not err)
 	    (run-hooks 'mew-real-post-hook)
@@ -909,7 +919,7 @@ Set privacy service which will be effective when \\[mew-draft-make-message]."
 	  (switch-to-buffer buf)
 	  (delete-other-windows))))))
 
-(defun mew-draft-remove-illegal-fields ()
+(defun mew-draft-remove-invalid-fields ()
   (when (mew-header-end)
     (save-excursion
       (save-restriction
