@@ -109,7 +109,7 @@
 ;;; Header sanity check
 ;;;
 
-(defun mew-encode-remove-illegal-fields ()
+(defun mew-encode-remove-invalid-fields ()
   (when (mew-header-end)
     (save-excursion
       (save-restriction
@@ -137,6 +137,7 @@
 	(setq folder (mew-canonicalize-folder (car folders)))
 	(setq folders (cdr folders))
 	(when (and mew-ask-fcc
+		   (mew-folder-localp folder)
 		   (not (file-directory-p (mew-expand-folder folder))))
 	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
 	      (mew-local-folder-check folder)
@@ -317,7 +318,7 @@
     ;; To:
     ;; Content-*
     ;; ----
-    (mew-header-clear) ;; mew-header-p returns nil
+    (mew-header-clear) ;; mew-in-header-p() returns nil
     ;; To:
     ;; Content-*
     (insert "\n"))
@@ -325,7 +326,7 @@
     ;; To:
     ;; ----
     ;; Content-*
-    (mew-header-clear) ;; mew-header-p returns nil
+    (mew-header-clear) ;; mew-in-header-p returns nil
     ;; To:
     ;; Content-*
     ))
@@ -364,7 +365,7 @@
   (widen)
   (let (multip recipients msgid-logtime)
     (goto-char (point-min))
-    (mew-encode-remove-illegal-fields)
+    (mew-encode-remove-invalid-fields)
     (mew-smtp-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;; Destination check
@@ -404,7 +405,7 @@
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
-	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
 	    (mew-encode-make-multi)
@@ -435,7 +436,7 @@
   (widen)
   (let (multip newsgroups msgid-logtime)
     (goto-char (point-min))
-    (mew-encode-remove-illegal-fields)
+    (mew-encode-remove-invalid-fields)
     (mew-nntp2-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;;
@@ -470,7 +471,7 @@
     (unless headerp
       (mew-encode-set-privacy pnm privacy)
       (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
-	(goto-char (mew-header-end)) ;; due to illegal null lines in the header
+	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
 	    (mew-encode-make-multi)
@@ -535,7 +536,7 @@
 		     (symbol-name cs)))))
 	(mew-frwlet
 	 (or cs mew-cs-autoconv) mew-cs-dummy
-	 (insert-file-contents file)))
+	 (mew-insert-file-contents file)))
       ;; A user may specify charset to convery JIS X 0201 Katakana.
       ;; So, we need to avoid the sanity check.
       (unless charset
@@ -583,7 +584,7 @@
       (unless textp
 	(mew-frwlet
 	 mew-cs-text-for-read mew-cs-dummy
-	 (insert-file-contents file)))
+	 (mew-insert-file-contents file)))
       (when (and (string= ct mew-ct-msg)
 		 (or mew-inherit-7bit (not mew-use-8bit))
 		 (mew-case-equal cte mew-8bit))
@@ -596,7 +597,7 @@
 	(mew-frwlet
 	 (if linebasep mew-cs-text-for-read mew-cs-binary)
 	 mew-cs-dummy
-	 (insert-file-contents file)))
+	 (mew-insert-file-contents file)))
       (when linebasep
 	(goto-char beg)
 	(mew-lf-to-crlf))
@@ -735,7 +736,8 @@
   (unless string (setq string "Next_Part"))
   (format mew-default-boundary
 	  string
-	  (mew-replace-character (current-time-string) 32 ?_)
+	  (mew-replace-character
+	   (mew-replace-character (current-time-string) 32 ?_) ?: ?_)
 	  (mew-random-string 3 t)))
 
 (defun mew-encode-multipart (syntax path depth &optional buffered)
@@ -1142,7 +1144,7 @@
     (when (file-exists-p back)
       (mew-frwlet
        mew-cs-m17n mew-cs-dummy
-       (insert-file-contents back)))))
+       (mew-insert-file-contents back)))))
 
 (defun mew-encode-undo ()
   "Get back to the draft before making MIME message."
