@@ -72,7 +72,7 @@ To slow down, PTY should be used.")
   (require 'mew-key)
   (if (string< emacs-version "21")
       (require 'mew-temacs)
-    (if (eq window-system 'x) (setq mew-icon-p t))
+    (if (featurep 'tool-bar) (setq mew-icon-p t))
     (setq mew-gemacs-p t)
     (require 'image) ;; to prepend the loading messages
     (require 'jit-lock) ;; to prepend the loading messages
@@ -134,10 +134,10 @@ To slow down, PTY should be used.")
   (defsubst mew-mouse-region-p () nil))
 
 (cond
- ((boundp 'auto-hscroll-mode) ;; Emacs 21.3 or later
+ ((boundp 'auto-hscroll-mode) ;; Emacs 21.3.50 or later
   (defsubst mew-hscroll ()
     (set (make-local-variable 'auto-hscroll-mode) t)))
- ((boundp 'automatic-hscrolling) ;; Emacs 21.2 or earlier
+ ((boundp 'automatic-hscrolling) ;; Emacs 21.3 or earlier
   (defsubst mew-hscroll ()
     (set (make-local-variable 'automatic-hscrolling) t)))
  (t
