@@ -313,6 +313,12 @@ in the context of FUNC."
     (setq string (replace-match " " nil t string)))
   string)
 
+(defun mew-replace-white-space2 (string)
+  "Replace white characters to under score."
+  (while (string-match "[\n\t ]+" string)
+    (setq string (replace-match "_" nil t string)))
+  string)
+
 (defsubst mew-capitalize (ostr)
   "Syntax table independent version of capitalize.
 Words are separated by '/' and '-'."
@@ -356,7 +362,7 @@ Words are separated by '/' and '-'."
 			 buffer-file-coding-system)))
 	(mew-frwlet
 	 rcs mew-cs-dummy
-	 (insert-file-contents file nil 0 size))
+	 (mew-insert-file-contents file nil 0 size))
 	(if (boundp 'buffer-file-coding-system)
 	    (setq buffer-file-coding-system old-cs)))
       ;; return physical size
@@ -456,8 +462,8 @@ also ignored."
 
 (defun mew-folder-case (folder)
   ;; "case" must be distinguished from drive
-  (if (string-match mew-regex-case folder)
-      (cons (mew-match-string 1 folder) (mew-match-string 2 folder))))
+  (if (string-match mew-regex-case1 folder)
+      (cons (mew-match-string 1 folder) (mew-match-string 3 folder))))
 
 (defun mew-folder-folder (folder)
   (let ((case-folder (mew-folder-case folder)))
@@ -554,10 +560,15 @@ also ignored."
 ;;
 
 (defun mew-canonicalize-folder (folder)
-  (if (or (mew-folder-localp folder)
-	  (file-name-absolute-p folder))
-      folder
-    (mew-string-to-local folder)))
+  (cond
+   ((mew-folder-localp folder)
+    folder)
+   ((mew-folder-imapp folder)
+    folder)
+   ((file-name-absolute-p folder)
+    folder)
+   (t
+    (mew-string-to-local folder))))
 
 (defun mew-path-to-folder (path)
   (let ((regex (concat "^" (regexp-quote (file-name-as-directory (expand-file-name mew-mail-path))))))
@@ -568,37 +579,36 @@ also ignored."
 (defun mew-canonicalize-case-folder (case:folder)
   (let* ((case-folder (mew-folder-case case:folder))
 	 (case (car case-folder))
-	 (folder (cdr case-folder)))
+	 (folder (cdr case-folder))
+	 len)
     (unless folder (setq folder case:folder))
     (cond
-     ((mew-folder-localp folder) folder)
-     ((mew-folder-absolutep folder) folder)
+     ((mew-folder-localp folder)   (directory-file-name folder))
+     ((mew-folder-absolutep folder)(directory-file-name folder))
      ((mew-folder-virtualp folder) folder)
-     (case
-      (if (string-equal case mew-case-default)
-	  folder
-	case:folder))
-     (t case:folder))))
-
-(defun mew-folder-remote-to-local (folder)
-  (let ((case-folder (mew-folder-case folder))
-	case fld)
-    (when case-folder
-      (setq case (car case-folder))
-      (setq folder (cdr case-folder)))
-    (setq fld (mew-folder-string folder))
-    (cond
-     ((mew-folder-localp folder) folder)
-     ((mew-folder-popp folder)
-      (mew-concat-folder (mew-pop-folder case) fld))
-     ((mew-folder-imapp folder)
-      (mew-concat-folder (mew-imap-folder case) fld))
-     ((mew-folder-nntpp folder)
-      (mew-concat-folder (mew-nntp-folder case) fld)))))
+     (t
+      (if (null case)
+	  (setq folder case:folder)
+	(if (string-equal case mew-case-default)
+	    (progn
+	      (setq case nil)
+	      (setq case:folder folder))))
+      (cond
+       ((mew-folder-popp folder) case:folder)
+       ((mew-folder-imapp folder)
+	(mew-imap-directory-file-name
+	 case:folder (or case ;; visit
+			 mew-inherit-case))) ;; refile
+       ((mew-folder-nntpp folder)
+	(setq len (1- (length case:folder)))
+	(if (char-equal (aref case:folder len) ?.)
+	    (substring case:folder 0 len)
+	  case:folder)))))))
 
 ;;
 
 (defun mew-expand-folder2 (folder)
+  "Expanding FOLDER to a relative path to '+'"
   (let ((case-folder (mew-folder-case folder))
 	case subdir)
     ;; The length of "case" must be longer than or equal to 2.
@@ -612,10 +622,12 @@ also ignored."
      ((mew-folder-nntpp folder)
       (mew-concat-folder (mew-nntp-folder case) subdir))
      ((mew-folder-imapp folder)
-      (mew-concat-folder (mew-imap-folder case) subdir case))
+      (mew-concat-folder (mew-imap-folder case)
+			 (mew-imap-expand-folder case subdir)))
      (t folder))))
 
 (defun mew-expand-folder (folder &optional message)
+  "Expanding FOLDER to its absolute path"
   (when (stringp folder)
     (let (dir)
     (setq folder (mew-expand-folder2 folder))
@@ -628,13 +640,12 @@ also ignored."
 	(expand-file-name message dir)
       dir))))
 
-(defun mew-concat-folder (folder subfolder &optional case)
+(defun mew-concat-folder (folder subfolder)
+  (concat (file-name-as-directory folder) subfolder))
+
+(defun mew-concat-folder2 (folder subfolder case)
   (if (mew-folder-imapp folder)
-      (let* ((sep (mew-imap-separator case))
-	     (regex (concat (regexp-quote sep) "$")))
-	(if (string-match regex folder)
-	    (concat folder subfolder)
-	  (concat folder sep subfolder)))
+      (concat (mew-imap-file-name-as-directory folder case) subfolder)
     (concat (file-name-as-directory folder) subfolder)))
 
 (defsubst mew-dir-messages (dir &optional regex)
@@ -657,6 +668,14 @@ also ignored."
     (mew-alet
      (directory-files edir nil regex 'no-sort))))
 
+(defun mew-folder-messages (folder)
+  (let* ((dir (mew-expand-folder folder))
+	 msgs nums)
+    (when (file-directory-p dir)
+      (setq msgs (mew-dir-messages dir))
+      (setq nums (sort (mapcar 'string-to-int msgs) '<))
+      (mapcar 'int-to-string nums))))
+
 (defun mew-folder-new-message (folder &optional num-only cache)
   (let* ((dir (mew-expand-folder folder))
 	 (regex (if cache mew-regex-message-files3 mew-regex-message-files))
@@ -707,6 +726,43 @@ also ignored."
 	(write-region "touched by Mew." nil file nil 'no-msg)
 	(mew-set-file-modes file)))))
 
+;; this kind of defmacro can't recurse
+;; if +foo exists and we insert +foo/bar,
+;; +foo/ should be inserted, too. But +foo/ does not match to +foo...
+(defmacro mew-folder-insert (folder lst subdir)
+  `(unless (mew-assoc-equal ,folder ,lst 0)
+     (let ((case-fold-search nil)
+	   (max (1- (length ,lst)))
+	   (pair (mew-folder-func ,folder ,subdir))
+	   (min 0)
+	   mid crr prv)
+       (while (> (- max min) 20) ;; 20 is enough?
+	 (setq mid (/ (+ min max) 2))
+	 (if (string< (car (nth mid ,lst)) ,folder)
+	     (setq min mid)
+	   (setq max mid)))
+       (setq crr (nthcdr min ,lst))
+       (while (and crr (string< (car (car crr)) ,folder))
+	 (setq prv crr)
+	 (setq crr (cdr crr)))
+       (if prv
+	   (setcdr prv (cons pair crr))
+	 (setq ,lst (cons pair crr))))))
+
+(defmacro mew-folder-delete (folder lst)
+  `(setq ,lst (delete (assoc ,folder ,lst) ,lst)))
+
+(defun mew-folder-node-p (folder &optional case)
+  (cond
+   ((mew-folder-localp folder)
+    (let* ((dir (file-name-as-directory folder))
+	   (regex (concat "^" (regexp-quote dir))))
+      (mew-assoc-match regex (mew-local-folder-alist) 0)))
+   ((mew-folder-imapp folder)
+    (let* ((dir (mew-imap-file-name-as-directory folder case))
+	   (regex (concat "^" (regexp-quote dir))))
+      (mew-assoc-match regex (mew-imap-folder-alist case) 0)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Folder spec
@@ -877,6 +933,39 @@ Does not examine containing directories for links."
 (defun mew-delete-file (file)
   (if (and (stringp file) (file-exists-p file)) (delete-file file)))
 
+(defun mew-insert-file-contents (&rest args)
+  "A safe version of insert-file-contents.
+This checks -*-coding:ctext;-*- internally when including a file."
+  (let ((after-insert-file-functions nil))
+    ;; preventing after-insert-file-set-buffer-file-coding-system
+    (apply 'mew-insert-file-contents2 args)))
+
+(defun mew-insert-file-contents2 (&rest args)
+  "Mew version of insert-file-contents.
+This sets  buffer-file-coding-system."
+  (let ((auto-image-file-mode nil))
+    (apply 'insert-file-contents args)))
+
+(defun mew-find-file-noselect (&rest args)
+  "A safe version of find-file-noselect.
+This checks -*-coding:ctext;-*- internally when including a file.
+But this does not set buffer-file-coding-system."
+  (let ((after-insert-file-functions nil))
+    ;; preventing after-insert-file-set-buffer-file-coding-system
+    (apply 'mew-find-file-noselect2 args)))
+
+(defun mew-find-file-noselect2 (&rest args)
+  "A safe version of find-file-noselect.
+This checks -*-coding:ctext;-*- internally when including a file
+and sets buffer-file-coding-system."
+  (let ((auto-image-file-mode nil)
+	(format-alist nil)
+	(auto-mode-alist nil)
+        (enable-local-variables nil)
+	(find-file-hook nil)
+        (find-file-hooks nil))
+    (apply 'find-file-noselect args)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; temp name
@@ -960,21 +1049,31 @@ Does not examine containing directories for links."
    (buffer-disable-undo)))
 
 (defun mew-remove-buffer (buf)
-  (if (get-buffer buf) (kill-buffer buf)))
+  (if (and buf (get-buffer buf)) (kill-buffer buf)))
 
 (defmacro mew-elet (&rest body)
   `(let ((buffer-read-only nil)
 	 (inhibit-read-only t)
 	 (after-change-functions nil)
+	 (mark-active nil) ;; for Emacs
 	 (zmacs-regions nil)) ;; for XEmacs
      ,@body))
 
+(defun mew-push-mark ()
+  ;; for C-x C-x
+  (let ((mark-active nil) ;; for Emacs
+	(zmacs-regions nil)) ;; for XEmacs
+    (push-mark (point) t t)))
+
 (defsubst mew-region-bytes (beg end buf)
-  (if (fboundp 'string-bytes)
-      (save-excursion
-	(set-buffer buf)
-	(string-bytes (mew-buffer-substring beg end)))
-    (- end beg)))
+  ;; string-bytes() acts differently on each Emacs.
+  ;; set-buffer-multibyte is also buggy.
+  ;; So, use this way.
+  (save-excursion
+    (set-buffer buf)
+    (if (fboundp 'string-as-unibyte)
+	(length (string-as-unibyte (mew-buffer-substring beg end)))
+      (- end beg))))
 
 (defun mew-count-lines (beg end)
   "Return number of lines between BEG and END."
@@ -1197,7 +1296,7 @@ from (mew-user), (mew-mail-address), and 'mew-mail-address-list'."
 	  (mew-erase-buffer)
 	  (mew-frwlet
 	   mew-cs-m17n mew-cs-dummy
-	   (insert-file-contents fullname))
+	   (mew-insert-file-contents fullname))
 	  (setq lisp 
 		(condition-case nil
 		    (read (current-buffer))
