@@ -437,27 +437,27 @@ Remove anonymous addresses."
 ;;; Functions for ID
 ;;;
 
-(defun mew-idstr-get-first-id (idstr &optional no-angle)
+(defun mew-idstr-get-first-id (idstr)
   (when (stringp idstr)
-    (let ((n (if no-angle 1 0)))
-      (if (string-match mew-regex-id idstr)
-	  (match-string n idstr)))))
+    (if (string-match mew-regex-id idstr)
+	(mew-replace-white-space2 (match-string 0 idstr)))))
 
-(defun mew-idstr-get-last-id (idstr &optional no-angle)
+(defun mew-idstr-get-last-id (idstr)
   (when (stringp idstr)
-    (let ((n (if no-angle 1 0)) beg end start)
+    (let (beg end start)
       (while (string-match mew-regex-id idstr start)
-	(setq beg (match-beginning n))
-	(setq end (match-end n))
+	(setq beg (match-beginning 0))
+	(setq end (match-end 0))
 	(setq start (match-end 0)))
-      (if (and beg end) (substring idstr beg end)))))
+      (if (and beg end)
+	  (mew-replace-white-space2 (substring idstr beg end))))))
 
-(defun mew-idstr-to-id-list (idstr &optional no-angle rev)
+(defun mew-idstr-to-id-list (idstr &optional rev)
   (when (stringp idstr)
-    (let ((n (if no-angle 1 0)) ret start)
+    (let (ret start)
       (while (string-match mew-regex-id idstr start)
-	(setq ret (cons (match-string n idstr) ret))
-	(setq start (match-end 0)))
+	(setq start (match-end 0))
+	(setq ret (cons (mew-replace-white-space2 (match-string 0 idstr)) ret)))
       (if rev
 	  ret
 	(nreverse ret)))))
