@@ -170,48 +170,30 @@ each candidate is a directory in 'while' loop."
 ;;; Folder sub function
 ;;;
 
-(defun mew-local-folder-pair (folder)
-  (let* ((dir (directory-file-name (mew-folder-string folder)))
-	 ;; foo/bar  -> foo/bar
-	 ;; foo/bar/ -> foo/bar
-	 (subdir (file-name-nondirectory dir)))
-	 ;; foo/bar -> bar 
-	 ;; foo -> foo
-    (if (string-match mew-regex-ignore-folders folder)
-	(mew-folder-func folder)
-      (mew-folder-func folder subdir))))
-
 (defun mew-local-folder-insert (folder)
   "Insert FOLDER to 'mew-local-folder-alist'.
 Binary search is used for speed reasons."
-  (mew-local-folder-insert1 folder)
-  (if (and (string-match (concat "^" (regexp-quote mew-friend-folder)) folder)
-	   (not (member folder mew-local-friend-folder-list)))
-      (setq mew-local-friend-folder-list
-	    (cons folder mew-local-friend-folder-list)))
-  (mew-local-folder-save))
+  (let (dir subdir)
+    (unless (string-match mew-regex-ignore-folders folder)
+      (setq dir (directory-file-name (mew-folder-string folder)))
+      ;; foo/bar  -> foo/bar
+      ;; foo/bar/ -> foo/bar
+      (setq subdir (file-name-nondirectory dir))
+      ;; foo/bar -> bar 
+      ;; foo -> foo
+      )
+    (mew-folder-insert folder mew-local-folder-alist subdir)
+    (if (and (string-match (concat "^" (regexp-quote mew-friend-folder)) folder)
+	     (not (member folder mew-local-friend-folder-list)))
+	(setq mew-local-friend-folder-list
+	      (cons folder mew-local-friend-folder-list)))
+    (mew-local-folder-save)))
 
-(defun mew-local-folder-insert1 (folder)
-  (unless (mew-assoc-equal folder mew-local-folder-alist 0)
-    (let ((case-fold-search nil)
-	  (max (1- (length mew-local-folder-alist)))
-	  (min 0) mid crr prv parent)
-      (while (> (- max min) 20) ;; 20 is enough?
-	(setq mid (/ (+ min max) 2))
-	(if (string< (car (nth mid mew-local-folder-alist)) folder)
-	    (setq min mid)
-	  (setq max mid)))
-      (setq crr (nthcdr min mew-local-folder-alist))
-      (while (and crr (string< (car (car crr)) folder))
-	(setq prv crr)
-	(setq crr (cdr crr)))
-      (if prv
-	  (setcdr prv (cons (mew-local-folder-pair folder) crr))
-	(setq mew-local-folder-alist
-	      (cons (mew-local-folder-pair folder) crr)))
-      ;; registering its parent directory
-      (setq parent (file-name-directory (directory-file-name folder)))
-      (if parent (mew-local-folder-insert1 parent)))))
+(defun mew-local-folder-delete (folder)
+  (mew-folder-delete folder mew-local-folder-alist)
+  (setq mew-local-friend-folder-list
+	(delete folder mew-local-friend-folder-list))
+  (mew-local-folder-save))
 
 (defun mew-local-folder-check (folder &optional ask)
   (let ((absdir (mew-expand-folder folder)))
@@ -221,7 +203,9 @@ Binary search is used for speed reasons."
 	      (y-or-n-p (format "%s does not exist. Create it? " folder)))
 	  (progn
 	    (mew-make-directory absdir)
-	    (mew-local-folder-insert folder)
+	    ;; may be called by IMAP...
+	    (if (mew-folder-localp folder)
+		(mew-local-folder-insert folder))
 	    folder)
 	nil))))
 
@@ -231,7 +215,7 @@ Binary search is used for speed reasons."
 ;;;
 
 (defvar mew-local-info-list
-  '("directive" "bnm" "mkdb" "flush" "rcnt" "rttl" "first"))
+  '("directive" "bnm" "mdb" "flush" "rcnt" "rttl" "first"))
 
 (mew-info-defun "mew-local-" mew-local-info-list)
 
@@ -264,7 +248,7 @@ Binary search is used for speed reasons."
 (defun mew-local-retrieve (directive &rest args)
   ;; 'inc  folder opts
   ;; 'scan folder range
-  ;; 'vir  opts func lra
+  ;; 'vir  opts func lra erase
   (mew-summary-with-mewl
    (let* ((process-connection-type mew-connection-type-for-scan)
 	  (bnm (mew-summary-folder-name 'ext))
@@ -285,7 +269,7 @@ Binary search is used for speed reasons."
        (setq range (nth 1 args))
        (when (nth 2 args)
 	 ;; erasing
-	 (mew-local-set-mkdb pnm (mew-summary-mark-collect4))
+	 (mew-local-set-mdb pnm (mew-summary-mark-collect4))
 	 (mew-erase-buffer)
 	 (setq mew-summary-buffer-raw nil)
 	 (mew-summary-folder-cache-save))
@@ -349,31 +333,20 @@ Binary search is used for speed reasons."
        (mew-scan-body vec draftp)
        (mew-scan-insert-line bnm vec width nil mark)
        (delete-region (point-min) (point-max))
-       (sit-for 0) ;; for XEmacs
+       (mew-timing) ;; for XEmacs
        (process-send-string process "\n")))))
 
 (defun mew-local-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (bnm (mew-local-get-bnm pnm))
 	 (virtualp (mew-folder-virtualp bnm))
-	 (mdb (mew-local-get-mkdb pnm))
-	 (flush (mew-local-get-flush pnm))
-	 opos ent msg mrk)
+	 (mdb (mew-local-get-mdb pnm))
+	 (flush (mew-local-get-flush pnm)))
     (mew-local-debug "SENTINEL" event)
     (mew-filter
      (mew-set-buffer-multibyte t)
      (set-buffer bnm)
-     (setq opos (point))
-     (goto-char (point-min))
-     (while mdb
-       (setq ent (car mdb))
-       (setq mdb (cdr mdb))
-       (setq msg (car ent))
-       (setq mrk (cdr ent))
-       (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	 (mew-mark-put mrk)
-	 (forward-line)))
-     (goto-char opos)
+     (mew-summary-mark-recover mdb)
      (if (and virtualp (mew-vinfo-get-func))
 	 (funcall (mew-vinfo-get-func)))
      (mew-vinfo-set-func nil)
