@@ -78,9 +78,9 @@ If optional argument NO-MSG is non-nil, no message is displayed."
 	(setq fld (mew-summary-folder-name))
 	(setq msg (mew-summary-message-number))
 	(mew-mark-remove)
-	(if (mew-virtual-p)
-	    (mew-mark-alist-set alist fld msg)
-	  (if func (funcall func fld msg))) ;; in physical only
+	(when (mew-virtual-p)
+	  (mew-mark-alist-set alist fld msg))
+	(if func (funcall func fld msg))
 	(forward-line))
       (set-buffer-modified-p nil)
       (mew-summary-unmark-in-physical-alist alist func)))
@@ -183,6 +183,20 @@ cons pairs of folder name and message number."
 	(forward-line))
       (nreverse ret))))
 
+(defun mew-summary-mark-recover (mdb)
+  (let ((opos (point))
+	ent msg mrk)
+    (goto-char (point-min))
+    (while mdb
+      (setq ent (car mdb))
+      (setq mdb (cdr mdb))
+      (setq msg (car ent))
+      (setq mrk (cdr ent))
+      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(mew-mark-put mrk)
+	(forward-line)))
+    (goto-char opos)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Base function
@@ -210,10 +224,8 @@ See also mew-mark-afterstep-spec."
      ((eq action 2)
       (mew-summary-goto-message)
       (mew-decode-syntax-delete)
-      ;; for C-x C-x
       (beginning-of-line)
-      (let ((zmacs-regions nil))
-	(push-mark (point) t t))
+      (mew-push-mark)
       (mew-summary-display-after mew-summary-mark-direction)))))
 
 (defun mew-mark-put-mark (newmark &optional no-msg valid-only)
@@ -221,8 +233,7 @@ See also mew-mark-afterstep-spec."
 If NO-MSG is non-nil, no message is displayed.
 NO-MSG also means that this function is being called in loop."
   (mew-summary-msg-or-part
-   (let (oldmark oldlevel oldname newlevel newname case zmacs-regions
-	 msg fld marked validp)
+   (let (oldmark oldlevel oldname newlevel newname case msg fld marked validp)
      (save-excursion
        (mew-summary-goto-message)
        (when (mew-sumsyn-match mew-regex-sumsyn-short)
@@ -340,44 +351,46 @@ original position."
        (setq ,alist (cons (list ,fld imsg) ,alist)))))
 
 (defun mew-summary-mark-in-physical-alist (alist mark &optional func)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
-      (setq fld (car ent))
-      (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
-      (when (get-buffer fld)
-	(set-buffer fld)
-	(save-excursion
-	  (goto-char (point-min))
-	  (while msgs
-	    (setq msg (int-to-string (car msgs)))
-	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	      (if func (funcall func))
-	      (mew-mark-put mark)
-	      (forward-line)))
-	  (set-buffer-modified-p nil))))))
+  (save-excursion
+    (let (ent fld msg msgs)
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq fld (car ent))
+	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
+	(when (get-buffer fld)
+	  (set-buffer fld)
+	  (save-excursion
+	    (goto-char (point-min))
+	    (while msgs
+	      (setq msg (int-to-string (car msgs)))
+	      (setq msgs (cdr msgs))
+	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		(if func (funcall func))
+		(mew-mark-put mark)
+		(forward-line)))
+	    (set-buffer-modified-p nil)))))))
 
 (defun mew-summary-unmark-in-physical-alist (alist func)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
-      (setq fld (car ent))
-      (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
-      (when (get-buffer fld)
-	(set-buffer fld)
-	(save-excursion
-	  (goto-char (point-min))
-	  (while msgs
-	    (setq msg (int-to-string (car msgs)))
-	    (setq msgs (cdr msgs))
-	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	      (mew-mark-remove)
-	      (if func (funcall func fld msg))
-	      (forward-line)))
-	  (set-buffer-modified-p nil))))))
+  (save-excursion
+    (let (ent fld msg msgs)
+      (while alist
+	(setq ent (car alist))
+	(setq alist (cdr alist))
+	(setq fld (car ent))
+	(setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
+	(when (get-buffer fld)
+	  (set-buffer fld)
+	  (save-excursion
+	    (goto-char (point-min))
+	    (while msgs
+	      (setq msg (int-to-string (car msgs)))
+	      (setq msgs (cdr msgs))
+	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		(mew-mark-remove)
+		(if func (funcall func fld msg))
+		(forward-line)))
+	    (set-buffer-modified-p nil)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -493,13 +506,14 @@ BEG and END."
     (save-excursion
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
-	(when (mew-sumsyn-match mew-regex-sumsyn-short)
-	  (when (mew-virtual-p)
-	    (setq fld (mew-sumsyn-folder-name))
-	    (setq msg (mew-sumsyn-message-number))
-	    (mew-mark-alist-set alist fld msg))
-	  (mew-summary-mark-as mark)
-          (setq n (1+ n)))
+	(unless (mew-summary-marked-p)
+	  (when (mew-sumsyn-match mew-regex-sumsyn-short)
+	    (when (mew-virtual-p)
+	      (setq fld (mew-sumsyn-folder-name))
+	      (setq msg (mew-sumsyn-message-number))
+	      (mew-mark-alist-set alist fld msg))
+	    (mew-mark-put mark)
+	    (setq n (1+ n))))
 	(forward-line))
       (set-buffer-modified-p nil))
     (mew-summary-mark-in-physical-alist alist mark)
