@@ -12,7 +12,7 @@
 ;;; Message info
 ;;;
 
-(defvar mew-minfo-list '("summary" "overlay"))
+(defvar mew-minfo-list '("summary" "eom" "veil-to" "veil-cc"))
 
 (mew-blinfo-defun 'mew-minfo mew-minfo-list)
 
