@@ -311,8 +311,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 		(if (not (re-search-forward (mew-pgp-get mew-pgp-msg-key-id) nil t))
 		    (setq keyid "not found")
 		  ;; xxx
-		  (setq keyid (mew-buffer-substring
-			       (match-beginning 1) (match-end 1)))
+		  (setq keyid (mew-match-string 1))
 		  (if (eq mew-pgp-ver mew-pgp-ver2) ;; xxx
 		      (setq keyid (concat "0x" keyid))))
 		(setq ret (concat mew-pgp-result-pubkey " ID = " keyid)))
@@ -738,7 +737,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (setq type (mew-pgp-check-old-pgp))
 	  (unless type
 	    (with-temp-buffer
-	      (insert-file-contents file)
+	      (mew-insert-file-contents file)
 	      (setq type (mew-pgp-check-old-pgp))))
 	  (if (null type)
 	      (message "No PGP message was found")
@@ -763,7 +762,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	    (when (file-exists-p file1)
 	      (mew-frwlet
 	       mew-cs-text-for-read mew-cs-dummy
-	       (insert-file-contents file1))
+	       (mew-insert-file-contents file1))
 	      (mew-delete-file file1))
 	    (if mew-mule-p
 		(mew-cs-decode-region (point) (point-max) mew-cs-autoconv))
@@ -889,69 +888,77 @@ public keyring."
 ;;; key fetch
 ;;;
 
-(defvar mew-pgp-fetch-process nil)
+(defun mew-pgp-fetch-key-by-xuri ()
+  (let ((key-list mew-x-pgp-key-list)
+	key val uri)
+    (save-excursion
+      (set-buffer (mew-buffer-message))
+      (catch 'loop
+	(while key-list
+	  (setq key (car key-list))
+	  (setq key-list (cdr key-list))
+	  (setq val (mew-header-get-value key))
+	  (when (and val (string-match "http:[^ \t\n]*" val))
+	    (setq val (mew-match-string 0 val))
+	    (if (y-or-n-p (format "Fetch a PGP public key from %s? " val))
+		(throw 'loop (setq uri val)))))))
+    (if uri
+	(mew-pgp-fetch uri)
+      (message "A PGP key is not fetched"))))
 
 (defun mew-pgp-fetch-key (arg)
-  "Fetch PGP public key."
+  "Fetch a PGP public key. 
+
+A PGP public key is fetch from one of the PGP PKS servers wich a
+key ID which is extracted from the X-Mew: field. The list of the
+PGP PKS servers is specified by 'mew-pgp-pks-servers'.
+
+If called with \\[universal-argument], a PGP public key is fetched 
+according to a URL in a field specified by 'mew-x-pgp-key-list'."
   (interactive "P")
-  (if (processp mew-pgp-fetch-process)
-      (if (y-or-n-p "Fetching process is running. Kill the process? ")
-	  (mew-pgp-fetch-process-kill))
-    (let (key key-list keyid xurl xurl-list xmew uri userid)
+  (if arg
+      (mew-pgp-fetch-key-by-xuri)
+    (let (keyid userid xmew)
       (save-excursion
 	(set-buffer (mew-buffer-message))
 	(setq userid (mew-header-parse-address mew-from:))
-	(cond
-	 (arg
-	  (setq key-list mew-x-pgp-key-list)
-	  (setq key (car key-list))
-	  (while key
-	    (setq xurl (mew-header-get-value key))
-	    (when (and xurl (string-match "http:[^ \t\n]*" xurl))
-	      (setq xurl (substring xurl (match-beginning 0) (match-end 0)))
-	      (setq xurl-list (append xurl-list (list xurl))))
-	    (setq key-list (cdr key-list))
-	    (setq key (car key-list))))
-	 (t
-	  (setq xmew (mew-header-get-value mew-x-mew:)))))
+	(setq xmew (mew-header-get-value mew-x-mew:)))
       (cond
-       ((and arg xurl-list)
-	(setq xurl (car xurl-list))
-	(while (and xurl (not uri))
-	  (if (y-or-n-p (format "fetch from %s? " xurl))
-	      (setq uri xurl))
-	  (setq xurl-list (cdr xurl-list))
-	  (setq xurl (car xurl-list))))
        ((and xmew
 	     (string-match "key. ID = \\(0x[0-9a-fA-F]+\\)" xmew nil)
-	     (setq keyid (substring xmew (match-beginning 1) (match-end 1))))
-	(if (y-or-n-p (format "Fetch Key ID=%s? " keyid))
-	    (setq uri (format mew-pgp-keyserver-url-template keyid))))
-       ((and userid (y-or-n-p (format "fetch User ID=%s? " userid)))
-	(setq uri (format mew-pgp-keyserver-url-template userid))))
-      (if uri
-	  (mew-pgp-fetch uri)
-	(message "Cannot find PGP public key's information")))))
+	     (setq keyid (mew-match-string 1 xmew)))
+	(if (y-or-n-p (format "Fetch the PGP public key for %s? " keyid))
+	    (mew-pgp-fetch-key-from-pks keyid)))
+       (userid
+	(if (y-or-n-p (format "Fetch the PGP public key for %s? " userid))
+	    (mew-pgp-fetch-key-from-pks userid)))
+       (t
+	(message "A PGP key is not fetched"))))))
+
+(defun mew-pgp-fetch-key-from-pks (&optional id)
+  (interactive)
+  (let* ((alist mew-pgp-pks-servers)
+	 (server (car (car alist)))
+	 uri)
+    (unless id
+      (setq id (read-string "PGP key id or e-mail address: ")))
+    (setq server (completing-read "PGP ker server: " alist nil nil server))
+    (unless (mew-pgp-pks-http-port-p server alist)
+      (setq server (concat server ":" mew-pgp-pks-port)))
+    (setq uri (concat "http://" server mew-pgp-pks-lang id))
+    (mew-pgp-fetch uri)))
 
 (defun mew-pgp-fetch (uri)
   (message "PGP key fetching...")
-  (mew-piolet
-   mew-cs-autoconv mew-cs-autoconv
-   (setq mew-pgp-fetch-process
-	 (apply 'start-process
-		"Mew PGP fetch"
-		(generate-new-buffer mew-buffer-prefix)
-		mew-prog-pgpkey
-		(append mew-prog-pgpkey-arg (list uri)))))
-  (set-process-sentinel mew-pgp-fetch-process 'mew-pgp-fetch-process-sentinel))
-
-(defun mew-pgp-fetch-process-kill ()
-  "Kill the current process fetching PGP keys."
-  (interactive)
-  (unwind-protect
-      (if (processp mew-pgp-fetch-process)
-	  (kill-process mew-pgp-fetch-process))
-    (setq mew-pgp-fetch-process nil)))
+  (let (pro)
+    (mew-piolet
+     mew-cs-autoconv mew-cs-autoconv
+     (setq pro (apply 'start-process
+		      "Mew PGP fetch"
+		      (generate-new-buffer mew-buffer-prefix)
+		      mew-prog-pgpkey
+		      (append mew-prog-pgpkey-args (list uri)))))
+    (set-process-sentinel pro 'mew-pgp-fetch-process-sentinel)))
 
 (defun mew-pgp-fetch-process-sentinel (process event)
   (message "PGP key fetching...done")
@@ -963,8 +970,7 @@ public keyring."
 	       (search-forward mew-pgp-key-end nil t))
 	  (mew-mime-pgp-keys-ext (current-buffer) (point-min) (point-max))
 	(message "PGP key fetch failed")))
-    (mew-remove-buffer buf)
-    (setq mew-pgp-fetch-process nil)))
+    (mew-remove-buffer buf)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; <Decrypt> or <Decrypt-then-Verify>
