@@ -16,72 +16,80 @@
   "Pick messages according to a pick pattern which you input, 
 then put the '*' mark onto them. If called with '\\[universal-argument]',
 execute 'mew-summary-pick-with-external'. Otherwise, 
-'mew-summary-pick-with-mewls' is called."
+'mew-summary-pick-with-mewl' is called."
   (interactive "P")
   (mew-summary-or-thread
    (let ((folder (mew-summary-folder-name))
-	 (pattern (if arg (mew-input-grep-pattern) (mew-input-pick-pattern)))
 	 (preline 0)
-	 msgs msgsback threadmsgs total linenum n)
-     (mew-sinfo-set-find-key nil) ;; force to ask a user
-     (message "Picking messages in %s..." folder)
-     (if arg
-	 (setq msgs (mew-summary-pick-with-external folder pattern))
-       (setq msgs (mew-summary-pick-with-mewls folder pattern)))
-     (setq msgsback msgs)
-     (message "Picking messages in %s...done" folder)
-     (if (null msgs)
-	 (message "No message to be marked")
-       (setq n (length msgs))
-       (if (= n 1)
-	   (message "Marking 1 message...")
-	 (message "Marking %d messages..." n))
+	 pattern msgs msgsback threadmsgs total linenum n)
+     (unless folder
+       ;; probably on a thread separator
        (save-excursion
-	 ;; Thread folder
-	 (when (mew-thread-p)
-	   (setq threadmsgs (mew-summary-thread-get-msglst
-			     (mew-vinfo-get-top) 'separator))
-	   (setq total (length threadmsgs))
-	   (goto-char (point-min))
-	   (while (and msgs (not (eobp)))
-	     (setq linenum (member (car msgs) threadmsgs))
-	     (if (null linenum)
-		 (setq msgsback (delete (car msgs) msgsback))
-	       (setq linenum (- total (length linenum)))
-	       (forward-line (- linenum preline))
-	       (setq preline linenum)
-	       (mew-summary-mark-as mew-mark-review))
-	     (setq msgs (cdr msgs)))
-	   (set-buffer-modified-p nil))
-	 ;; Physical folder
-	 (when (get-buffer folder)
-	   (set-buffer folder)
-	   (save-excursion
+	 (goto-char (point-min))
+	 (setq folder (mew-summary-folder-name))))
+     (if (null folder)
+	 (message "No message to be picked")
+       (if arg
+	   (progn
+	     (setq pattern (mew-input-grep-pattern))
+	     (mew-sinfo-set-find-key pattern))
+	 (setq pattern (mew-input-pick-pattern))
+	 (mew-sinfo-set-find-key nil)) ;; force to ask a user
+       (message "Picking messages in %s..." folder)
+       (if arg
+	   (setq msgs (mew-summary-pick-with-external folder pattern))
+	 (setq msgs (mew-summary-pick-with-mewl folder pattern)))
+       (setq msgsback msgs)
+       (message "Picking messages in %s...done" folder)
+       (if (null msgs)
+	   (message "No message to be marked")
+	 (setq n (length msgs))
+	 (if (= n 1)
+	     (message "Marking 1 message...")
+	   (message "Marking %d messages..." n))
+	 (save-excursion
+	   ;; Thread folder
+	   (when (mew-thread-p)
+	     (setq threadmsgs (mew-summary-thread-get-msglst
+			       (mew-vinfo-get-top) 'separator))
+	     (setq total (length threadmsgs))
 	     (goto-char (point-min))
-	     (setq msgs msgsback)
 	     (while (and msgs (not (eobp)))
-	       (when (re-search-forward (mew-regex-jmp-msg (car msgs)) nil t)
-		 (mew-summary-mark-as mew-mark-review)
-		 (forward-line))
+	       (setq linenum (member (car msgs) threadmsgs))
+	       (if (null linenum)
+		   (setq msgsback (delete (car msgs) msgsback))
+		 (setq linenum (- total (length linenum)))
+		 (forward-line (- linenum preline))
+		 (setq preline linenum)
+		 (mew-summary-mark-as mew-mark-review))
 	       (setq msgs (cdr msgs)))
-	     (set-buffer-modified-p nil))))
-       (if (= n 1)
-	   (message "Marking 1 message...done")
-	 (message "Marking %d messages...done" n))))))
+	     (set-buffer-modified-p nil))
+	   ;; Physical folder
+	   (when (get-buffer folder)
+	     (set-buffer folder)
+	     (save-excursion
+	       (goto-char (point-min))
+	       (setq msgs msgsback)
+	       (while (and msgs (not (eobp)))
+		 (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
+		   (mew-summary-mark-as mew-mark-review)
+		   (forward-line))
+		 (setq msgs (cdr msgs)))
+	       (set-buffer-modified-p nil))))
+	 (if (= n 1)
+	     (message "Marking 1 message...done")
+	   (message "Marking %d messages...done" n)))))))
 
-(defun mew-summary-pick-with-mewls (folder pattern)
-  "A function to pick messages matching PATTERN with 'mewls'"
-  (mew-summary-with-mewls
-   (let ((buf (generate-new-buffer mew-buffer-prefix))
-	 (fld (mew-expand-folder2 folder))
+(defun mew-summary-pick-with-mewl (folder pattern)
+  "A function to pick messages matching PATTERN with 'mewl'"
+  (mew-summary-with-mewl
+   (let ((fld (mew-expand-folder2 folder))
 	 msgs)
-     (save-excursion
-       (set-buffer buf)
-       (mew-erase-buffer)
+     (with-temp-buffer
        (mew-set-buffer-multibyte t)
        (mew-piolet
 	mew-cs-text-for-read mew-cs-text-for-write
-	(call-process mew-prog-mewls nil t nil
+	(call-process mew-prog-mewl nil t nil
 		      "-b" mew-mail-path
 		      "-l" (int-to-string mew-scan-max-field-length)
 		      "-p" pattern "-s" fld))
@@ -90,7 +98,6 @@ execute 'mew-summary-pick-with-external'. Otherwise,
 	 (if (looking-at mew-regex-message-files)
 	     (setq msgs (cons (mew-match-string 0) msgs)))
 	 (forward-line)))
-     (mew-remove-buffer buf)
      (nreverse msgs))))
 
 (defvar mew-summary-pick-with-external-function
@@ -118,14 +125,10 @@ This function MUST return text in which each line begins with a file name.")
 'mew-summary-pick-with-external-function'."
   (let* ((dir (mew-expand-folder folder))
 	 (default-directory dir) ;; buffer local
-	 (msgs (mew-dir-messages "."))
-	 buf)
+	 (msgs (mew-dir-messages ".")))
     ;; no sort here
     (when msgs
-      (save-excursion
-	(setq buf (generate-new-buffer mew-buffer-prefix))
-	(set-buffer buf)
-	(mew-erase-buffer)
+      (with-temp-buffer
 	(cd dir)
 	(mew-piolet
 	 mew-cs-text-for-read mew-cs-text-for-write
@@ -137,7 +140,6 @@ This function MUST return text in which each line begins with a file name.")
 	  (if (looking-at mew-regex-message-files2)
 	      (setq msgs (cons (mew-match-string 0) msgs)))
 	  (forward-line)))
-      (mew-remove-buffer buf)
       (setq msgs (mew-uniq-list msgs))
       (setq msgs (mapcar 'string-to-int msgs))
       (setq msgs (sort msgs '<))
@@ -347,7 +349,7 @@ keyword, execute this command with '\\[universal-argument]'."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2002 Mew developing team.
+;; Copyright (C) 1996-2003 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
