@@ -136,11 +136,11 @@ mew-refile-guess-from-me-is-special."
       (setq mew-assoc-folder 'mew-assoc-folder-fast)
     (setq mew-assoc-folder 'mew-assoc-folder-slow)))
 
-(defun mew-assoc-folder-slow (key alist localp)
+(defun mew-assoc-folder-slow (key alist sep)
   ;; Case-sensitive so you can use capital letters for your folders.
   ;; But this is slow.
   (let ((skey (downcase key))
- 	(regex (concat mew-path-separator "$"))
+	(regex (concat (regexp-quote sep) "$"))
 	ent name ret)
     (catch 'loop
       (while alist
@@ -148,42 +148,46 @@ mew-refile-guess-from-me-is-special."
  	(setq name (cdr ent))
  	(when (and (stringp name) (string= (downcase name) skey))
  	  (setq ret ent)
- 	  (unless (and localp (string-match regex (car ent)))
+ 	  (unless (string-match regex (car ent))
 	    (throw 'loop nil)))
  	(setq alist (cdr alist))))
     ret))
 
-(defun mew-assoc-folder-fast (key alist localp)
+(defun mew-assoc-folder-fast (key alist sep)
   ;; Case-insensitive so you cannot use capital letters for your folders.
   ;; But this is fast.
-  (let ((skey (downcase key)) ret ret2 regex)
-    (if (not localp)
-	(rassoc skey alist)
-      ;; ("+foo/" . "foo")
-      ;; ("+foo/foo" . "foo")
-      (setq regex (concat mew-path-separator "$"))
-      (setq ret (rassoc skey alist))
+  ;;
+  ;; ("+foo/" . "foo")
+  ;; ("+foo/foo" . "foo")
+  (let* ((skey (downcase key))
+	 (regex (concat (regexp-quote sep) "$"))
+	 (ret (rassoc skey alist))
+	 ret2)
+    (while (and ret (string-match regex (car ret)))
+      (setq alist (cdr (member ret alist)))
       (setq ret2 ret)
-      (while (and ret (string-match regex (car ret)))
-	(setq alist (cdr (member ret alist)))
-	(setq ret (rassoc skey alist)))
-      (or ret ret2))))
+      (setq ret (rassoc skey alist)))
+    (or ret ret2)))
 
 (defun mew-refile-guess-by-folder ()
   ;; Guess folders by the To:/Cc: field with folder alist.
   ;; Mainly used for mailing-list.
   (let* ((to-cc (mew-header-parse-address-list mew-refile-guess-key-list))
-	 (proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+	 (proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (alist (mew-proto-folder-alist proto case))
-	 (localp (mew-folder-localp proto))
-	 ent ret ml-addr ml-name)
+	 sep ent ret ml-addr ml-name)
+    (cond
+     ((mew-folder-imapp proto)
+      (setq sep (mew-imap-separator case)))
+     (t
+      (setq sep mew-path-separator)))
     (while to-cc
       (setq ml-addr (mew-addrstr-parse-address (or (car to-cc) "")))
       (when ml-addr
 	(setq ml-name (mew-addrstr-extract-user ml-addr))
-	(setq ent (or (funcall mew-assoc-folder ml-addr alist localp)
-		      (funcall mew-assoc-folder ml-name alist localp)))
+	(setq ent (or (funcall mew-assoc-folder ml-addr alist sep)
+		      (funcall mew-assoc-folder ml-name alist sep)))
 	(mew-addq ret (nth 0 ent)))
       (setq to-cc (cdr to-cc)))
     (nreverse ret)))
@@ -195,7 +199,7 @@ mew-refile-guess-from-me-is-special."
 
 (defun mew-refile-guess-by-alist ()
   ;; Guess folders by user-configured alist.
-  (mew-refile-guess-by-alist1 mew-refile-guess-alist))
+  (mew-refile-guess-by-alist1 (mew-refile-guess-alist mew-inherit-refile-case)))
 
 (defun mew-refile-guess-by-alist1 (rule)
   "Return a guessed folder according to the RULE. The syntax of
@@ -281,8 +285,8 @@ guessed values."
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-from-folder-body addr))
 
 (defun mew-refile-guess-by-from-folder-body (&optional addr)
-  (let* ((proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+  (let* ((proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (list (mew-proto-friend-folder-list proto case))
 	 (sep (if (mew-folder-imapp proto)
 		  (mew-imap-separator case)
@@ -319,14 +323,14 @@ guessed values."
   ;; Guess folders by the Newsgroups field with folder alist.
   (let* ((newsgroups (mew-addrstr-parse-value-list2 
 		     (mew-header-get-value mew-newsgroups:)))
-	 (proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
-	 (alist (mew-proto-folder-alist proto case))
+	 (proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
+	 (alist (mew-proto-folder-alist proto case)) ;; local folders
 	 newsgroup ent ret)
     (while newsgroups
       (setq newsgroup (car newsgroups))
       (setq newsgroups (cdr newsgroups))
-      (setq ent (funcall mew-assoc-folder newsgroup alist nil))
+      (setq ent (funcall mew-assoc-folder newsgroup alist mew-path-separator))
       (mew-addq ret (nth 0 ent)))
     (nreverse ret)))
 
@@ -340,13 +344,13 @@ guessed values."
   (mew-refile-guess-from-dispatch 'mew-refile-guess-by-default-body addr))
 
 (defun mew-refile-guess-by-default-body (&optional addr)
-  (let* ((proto (mew-sinfo-get-proto))
-	 (case (mew-sinfo-get-case))
+  (let* ((proto mew-inherit-refile-proto)
+	 (case mew-inherit-refile-case)
 	 (fld (mew-proto-friend-folder proto case))
 	 (from (downcase (or addr (mew-header-parse-address mew-from:) ""))))
     (if mew-refile-guess-strip-domainpart
 	(setq from (mew-addrstr-extract-user from)))
-    (mew-concat-folder fld from case)))
+    (mew-concat-folder2 fld from case)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -464,9 +468,10 @@ Return t if exists or created. Otherwise, return nil."
      ((mew-folder-nntpp folder) nil)
      ((mew-folder-virtualp folder) nil)
      ((mew-folder-imapp folder)
-      (let* ((case (mew-sinfo-get-case))
+      (let* ((case mew-inherit-refile-case)
 	     (mailboxes (mew-imap-folder-alist case)))
-	(if (assoc folder mailboxes)
+	(if (or (assoc folder mailboxes)
+		(assoc (mew-imap-file-name-as-directory folder case) mailboxes))
 	    t
 	  (if (and (y-or-n-p (format "%s does not exist. Create it? " folder))
 		   (mew-imap-folder-insert case folder))
@@ -497,14 +502,11 @@ Return t if exists or created. Otherwise, return nil."
   ;;  ((folder1 folder2...)
   ;;   (func1 guess11 guess12...)  ;; for learning
   ;;   (func2 guess12 guess22...))
-  (let ((proto (mew-proto-to-refile (mew-summary-folder-name 'ext)))
-	(case (mew-sinfo-get-case))
+  (let ((proto mew-inherit-refile-proto)
+	(case mew-inherit-refile-case)
 	learn-info folders ret cands singlep lst-lst lst fld)
     (save-excursion
       (set-buffer buf)
-      ;; copy to buffer local variables
-      (mew-sinfo-set-case case)
-      (mew-sinfo-set-proto proto)
       (setq learn-info (mew-refile-guess auto)))
     (if auto
 	;; if auto is set, simply use the guess.
@@ -603,27 +605,33 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 
 (defun mew-summary-refile-log (fld folders-str)
   (save-excursion
-    (let ((cbuf (current-buffer))
+    (beginning-of-line)
+    (let ((beg (point))
+	  (cbuf (current-buffer))
 	  (msg-id (mew-summary-my-id))
-	  (beg (point)) end)
+	  end)
       (forward-line)
       (when (search-backward "\r" nil t)
 	(setq end (point))
 	(set-buffer (get-buffer-create (mew-refile-log-buffer fld)))
 	(goto-char (point-max))
 	(mew-insert-buffer-substring cbuf beg end)
-	(insert "== src=" fld " dst=" folders-str " id=<" msg-id ">")
+	(insert "== src=" fld " dst=" folders-str " id=" msg-id)
 	(insert " date=" (mew-time-ctz-to-logtime (current-time)) "\n")))))
 
 (defun mew-summary-refile-body (&optional exp-flds auto no-msg no-mark exfld)
   (let ((pos (point))
-	msg folder folders cur-folders mark
-	buf learn-info fld tmp delbuf invalidp folders-str)
+	fld msg folder folders cur-folders mark
+	buf learn-info tmp delbuf invalidp folders-str
+	mew-inherit-refile-proto mew-inherit-refile-case case-folder)
     (mew-summary-goto-message)
     (when (mew-sumsyn-match mew-regex-sumsyn-short)
       (setq fld (mew-sumsyn-folder-name))
       (setq msg (mew-sumsyn-message-number)) ;; msg is never nil
       (setq invalidp (not (mew-msg-validp msg))))
+    (setq case-folder (mew-folder-case fld))
+    (setq mew-inherit-refile-case  (car case-folder))
+    (setq mew-inherit-refile-proto (or (cdr case-folder) fld))
     (setq mark (mew-summary-get-mark)) ;; any mark
     (cond
      ((and mark (> (mew-markdb-level mark) (mew-markdb-level mew-mark-refile)))
@@ -682,14 +690,15 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
 	   ((mew-virtual-p)
 	    (save-excursion
 	      (set-buffer fld)
-	      (goto-char (point-min))
-	      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-		(mew-refile-reset msg)
-		(mew-refile-set msg folders)
-		(mew-summary-refile-log fld folders-str)
-		(mew-summary-refile-override-body folders-str)
-		(mew-mark-put mew-mark-refile)
-		(set-buffer-modified-p nil))))
+	      (save-excursion
+		(goto-char (point-min))
+		(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		  (mew-refile-reset msg)
+		  (mew-refile-set msg folders)
+		  (mew-summary-refile-log fld folders-str)
+		  (mew-summary-refile-override-body folders-str)
+		  (mew-mark-put mew-mark-refile)
+		  (set-buffer-modified-p nil)))))
 	   (t
 	    (mew-refile-reset msg)
 	    (mew-refile-set msg folders)
@@ -705,21 +714,23 @@ with '\\[universal-argument]', it displays how the refile rules work in Message
     folders)) ;; return value
 
 (defun mew-summary-refile-report ()
-  (let ((proto (mew-proto-to-refile (mew-summary-folder-name 'ext)))
-	(case (mew-sinfo-get-case))
-	(fld (mew-summary-folder-name))
-	(msg (mew-summary-message-number2))
-	(win (selected-window))
+  (let ((win (selected-window))
 	(customize-var '(mew-refile-ctrl-multi
 			 mew-refile-guess-key-list
 			 mew-refile-guess-strip-domainpart
 			 mew-refile-guess-from-me-is-special))
-	guess)
+	fld msg guess case-folder
+	mew-inherit-refile-case mew-inherit-refile-proto)
+    (mew-summary-goto-message)
+    (when (mew-sumsyn-match mew-regex-sumsyn-short)
+      (setq fld (mew-sumsyn-folder-name))
+      (setq msg (mew-sumsyn-message-number)))
+    (setq case-folder (mew-folder-case fld))
+    (setq mew-inherit-refile-case  (car case-folder))
+    (setq mew-inherit-refile-proto (or (cdr case-folder) fld))
     (with-temp-buffer
       (mew-insert-message fld msg mew-cs-autoconv
 			  mew-header-reasonable-size)
-      (mew-sinfo-set-case case)
-      (mew-sinfo-set-proto proto)
       (setq guess (mew-refile-guess nil t)))
     (mew-window-configure 'message)
     ;; message buffer
@@ -832,10 +843,12 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-review nil t)
-	    (setq last (mew-summary-refile-body
-			last nil 'no-msg nil (mew-summary-folder-name 'ext)))
-	    (forward-line))
+	  (catch 'loop
+	    (while (re-search-forward mew-regex-msg-review nil t)
+	      (setq last (mew-summary-refile-body
+			  last nil 'no-msg nil (mew-summary-folder-name 'ext)))
+	      (unless last (throw 'loop t))
+	      (forward-line)))
 	  (message "Mark copying...done")))))))
 
 (defun mew-summary-mark-refile ()
@@ -851,9 +864,11 @@ This is very convenient to refile all messages picked by '\\[mew-summary-search-
       (save-excursion
 	(save-window-excursion
 	  (goto-char (point-min))
-	  (while (re-search-forward mew-regex-msg-review nil t)
-	    (setq last (mew-summary-refile-body last nil 'no-msg))
-	    (forward-line))
+	  (catch 'loop
+	    (while (re-search-forward mew-regex-msg-review nil t)
+	      (setq last (mew-summary-refile-body last nil 'no-msg))
+	      (unless last (throw 'loop t))
+	      (forward-line)))
 	  (message "Mark refiling...done")))))))
 
 (provide 'mew-refile)
