@@ -55,7 +55,7 @@ effect to this function."
     (when mew-summary-form-mark-delete
       (if (mew-characterp mew-summary-form-mark-delete)
 	  (setq mark-delete mew-summary-form-mark-delete))
-      (when (setq id (mew-idstr-get-first-id (MEW-ID) 'no-angle))
+      (when (setq id (mew-idstr-get-first-id (MEW-ID)))
 	(if (member id (mew-sinfo-get-scan-id)) ;; in Summary mode
 	    (setq duplicated t)
 	  (mew-sinfo-set-scan-id (cons id (mew-sinfo-get-scan-id))))))
@@ -81,6 +81,7 @@ effect to this function."
   (let ((ct (MEW-CT))
 	(case-fold-search t))
     (cond
+     ((mew-scan-message-invalidp)                  "#")
      ((mew-scan-message-truncatedp)                "T")
      ((string-match "Multipart/Signed"    ct)      "S")
      ((string-match "Multipart/Encrypted" ct)      "E")
@@ -160,9 +161,8 @@ with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
   (let* ((func (if mew-addrbook-for-summary
 		   (mew-addrbook-func mew-addrbook-for-summary)))
 	 (raw (or (mew-addrstr-parse-address addr) ""))
-	 (nickname (if func (funcall func raw)))
 	 (rules mew-summary-form-extract-rule)
-	 rule ret)
+	 rule ret nickname)
     (catch 'matched
       (while rules
 	(setq rule (car rules))
@@ -177,7 +177,10 @@ with -4. See also 'mew-summary-form-size-0k' and 'mew-summary-form-size-huge'."
 	  (throw 'matched (setq ret (mew-chop (match-string 1 addr)))))
 	 ((eq rule 'address)
 	  (throw 'matched (setq ret raw)))
-	 ((and (eq rule 'nickname) nickname)
+	 ((and (eq rule 'nickname)
+	       ;; set nickname here for efficiency
+	       (or nickname
+		   (setq nickname (if func (funcall func raw)))))
 	  (throw 'matched (setq ret nickname)))
 	 ((and (stringp rule)
 	       (string-match rule addr))
@@ -269,7 +272,7 @@ of thread indent position is returned."
   (let (func width str rightp nopad)
     (if (stringp spec)
 	(progn
-	  (setq TOTAL (+ TOTAL (length spec))) ;; xxx length
+	  (setq TOTAL (+ TOTAL (string-width spec)))
 	  spec)
       (if (symbolp spec)
 	  (setq width 1 func spec)
@@ -286,17 +289,17 @@ of thread indent position is returned."
 	(when (fboundp func)
 	  (setq str (funcall func))
 	  (if rightp
-	      (if (<= (length str) width)
+	      (if (<= (string-width str) width)
 		  (format (format "%%%ds" width) str)
-		(setq TOTAL (+ (- TOTAL width) (length str)))
+		(setq TOTAL (+ (- TOTAL width) (string-width str)))
 		str) ;; width may exceed.
 	    (mew-substring str width nil nopad)))))))
 
-(defun mew-scan-get-line (mew-vec WIDTH fld)
-  (let* ((TOTAL 0)
+(defun mew-scan-get-line (mew-vec WIDTH)
+  (let* ((TOTAL 0) (fld "")
 	 (line (mapconcat 'mew-scan-get-piece (mew-sinfo-get-summary-form) ""))
 	 par-id my-id msg ld uid siz irt-list)
-    (setq my-id (or (mew-idstr-get-first-id (MEW-ID) 'no-angle) ""))
+    (setq my-id (or (mew-idstr-get-first-id (MEW-ID)) ""))
     ;; RFC 2822 says: the "In-Reply-To:" field may be used to identify
     ;; the message (or messages) to which the new message is a reply,
     ;; while the "References:" field may be used to identify a
@@ -309,12 +312,12 @@ of thread indent position is returned."
     ;; (1) The In-Reply-To contains one ID, use it.
     ;; (2) The References contains one or more IDs, use the last one.
     ;; (3) The In-Reply-To contains two or more IDs, use the first one.
-    (setq par-id (or (mew-idstr-get-first-id (MEW-XREF) 'no-angle) ""))
+    (setq par-id (or (mew-idstr-get-first-id (MEW-XREF)) ""))
     (when (string= par-id "")
-      (setq irt-list (mew-idstr-to-id-list (MEW-IRT) 'no-angle))
-      (if (length irt-list)
+      (setq irt-list (mew-idstr-to-id-list (MEW-IRT)))
+      (if (= (length irt-list) 1)
 	  (setq par-id (car irt-list))
-	(setq par-id (or (mew-idstr-get-last-id (MEW-REF) 'no-angle)
+	(setq par-id (or (mew-idstr-get-last-id (MEW-REF))
 			 (car irt-list)
 			 ""))))
     (when (mew-virtual-p)
@@ -323,7 +326,7 @@ of thread indent position is returned."
     (setq msg (mew-scan-get-message mew-vec))
     (setq uid (or (mew-scan-uid-uid (MEW-UID)) ""))
     (setq siz (or (mew-scan-uid-size (MEW-UID)) ""))
-    (setq ld (format "\r %s %s <%s> <%s> %s %s\n" fld msg my-id par-id uid siz))
+    (setq ld (format "\r %s %s %s %s %s %s\n" fld msg my-id par-id uid siz))
     (cons line ld)))
 
 ;; See also mew-summary-cook-region
@@ -331,7 +334,7 @@ of thread indent position is returned."
   (when (get-buffer folder)
     (save-excursion
       (set-buffer folder)
-      (let* ((line (mew-scan-get-line vec width folder))
+      (let* ((line (mew-scan-get-line vec width))
 	     (opos (point))
 	     (omax (point-max))
 	     beg med face olen nlen mark msg)
@@ -366,14 +369,14 @@ of thread indent position is returned."
 	   (mew-summary-refile-log folder mark-or-dst)
 	   (mew-summary-refile-override-body mark-or-dst)
 	   (mew-mark-put mew-mark-refile))
-	  ((integerp mark-or-dst)
-	   (mew-summary-mark-as mark-or-dst))
 	  (mark
 	   (mew-summary-mark-as mark))
-	  ((and mew-use-highlight-mark
+	  ((and mew-use-highlight-mark ;; mew-summary-form-mark
 		(setq mark (mew-summary-get-mark)) ;; duplicated, etc
 		(setq face (mew-highlight-mark-get-face mark)))
-	   (put-text-property beg med 'face face)))
+	   (put-text-property beg med 'face face))
+	  ((mew-characterp mark-or-dst) ;; mew-inbox-action-alist
+	   (mew-summary-mark-as mark-or-dst)))
 	 (if mew-use-highlight-mouse-line
 	     (put-text-property
 	      beg med 'mouse-face mew-highlight-mouse-line-face))
@@ -402,7 +405,8 @@ of thread indent position is returned."
 
 (defun mew-scan-header (&optional draftp)
   (let ((vec (make-vector (length mew-scan-fields) ""))
-	key med str n)
+	(lim (1- mew-scan-max-field-length))
+	(i 0) key med str n)
     (goto-char (point-min))
     (unless (re-search-forward mew-eoh nil t)
       (goto-char (point-max)))
@@ -414,75 +418,22 @@ of thread indent position is returned."
 	    (forward-line)
 	  (setq key (mew-capitalize (mew-match-string 1)))
 	  (setq med (match-end 0))
+	  ;; Three lines should be enough for Summary mode.
 	  (forward-line)
-	  (mew-header-goto-next)
-	  (when (setq n (mew-member-case-equal key mew-scan-fields))
-	    ;; xxx this may be able to tune up.
-	    (mew-header-decode-region key med (point) draftp)
+	  (while (and (< i lim) (looking-at mew-lwsp))
+	    (forward-line)
+	    (setq i (1+ i)))
+	  (when (and (setq n (mew-member-case-equal key mew-scan-fields))
+		     (string= (aref vec n) "")) ;; avoid multiple times
+	    (when (member key mew-scan-decode-fields)
+	      (mew-header-decode-region key med (point) draftp))
 	    ;; We need to keep composite properties of charset.
 	    ;; This must be "buffer-substring".
 	    (setq str (buffer-substring med (1- (point))))
-	    (aset vec n str)))))
+	    (aset vec n str))
+	  (mew-header-goto-next))))
     vec))
 
-(defvar mew-inbox-action-alist nil
-  "*This valuable controls actions when retrieving messages.
-Currently this works only when '\\<mew-summary-mode-map>\\[mew-summary-retrieve]' with POP or 'mewl/incm'.
-You can put any marks, typically 'o' and 'D' according to message
-headers.
-
-This value must be a list of entries. An entry is a list whose first
-element is a field defined in 'mew-scan-fields'. 
-
-If the second element is a list, the first element of the list must be
-a mark (e.g. ?D) or a string which is a comma separated folders (e.g
-\"+foo,+bar\" for ?o). Other elements of the list are regulaer
-expression to be matched to the value of the field.
-
-With the following configuration, Mew marks messages whose Subject:
-matches \"adult\" or \"money\" with 'D'. And Mew marks messages whose
-Subject is \"daily log\" with 'o' to be refiled \"+log\". Also Mew
-marks messages whose Content-Type: contains \"gb2312\" with 'D'.
-
-(setq mew-inbox-action-alist
-      '((\"Subject:\"
-	 (?D \"adult\" \"money\")
-	 (\"+log\" \"daily log\"))
-	(\"Content-Type:\"
-	 (?D \"gb2312\"))))
-
-If the second element of the entry is a symbol, it will be called as a
-function which returns nil or a mark or a string.
-
-If you are using 'spamassassin' which adds the \"X-Spam-Flag:\" field
-to messages and puts them to your mailbox, configure as follows:
-
-(defun mew-spam-assassin (val)
-  (let ((case-fold-search t))
-    (if (string-match \"yes\" val) ?D)))
-
-(setq mew-inbox-action-alist
-      '((\"X-Spam-Flag:\" mew-spam-assassin)))
-
-With this configuration, messages with the \"X-Spam-Flag:\" whose
-value is \"Yes\" are automatically marked with 'D' since the
-'mew-spam-assassin' function returns 'D' marks when the value is
-\"Yes\".
-
-If you are using 'bogofilter' which adds the \"X-Bogosity:\" field
-to messages and puts them to your mailbox, configure as follows:
-
-(setq mew-spam: \"X-Bogosity:\")
-
-(defun mew-spam-bogofilter (val)
-  (let ((case-fold-search t))
-    (if (string-match \"yes\" val) ?D)))
-
-(setq mew-inbox-action-alist
-      '((\"X-Bogosity:\" mew-spam-bogofilter)))
-")
-
-
 (defun mew-scan-field-to-func (key)
   (let ((n (mew-member-case-equal key mew-scan-fields))
 	(len (length mew-scan-fields-alias))
@@ -543,7 +494,7 @@ to messages and puts them to your mailbox, configure as follows:
 	 (cte (MEW-CTE))
 	 (body "")
 	 (case-fold-search t)
-	 textp ctl ct charset cs beg skip boundary found regex)
+	 textp charset cs beg skip boundary found regex)
     (catch 'break
       (cond
        (draftp
@@ -554,10 +505,13 @@ to messages and puts them to your mailbox, configure as follows:
 	(setq textp t)
 	(setq cs mew-cs-autoconv))
        (t
-	(setq ctl (mew-param-decode ctr))
-	(setq ct (mew-syntax-get-value ctl 'cap))
-	(when (and (string-match "^Multipart/" ct)
-		   (setq boundary (mew-syntax-get-param ctl "boundary")))
+	;; The following code is generic but too slow.
+	;; (setq ctl (mew-param-decode ctr))
+	;; (setq ct (mew-syntax-get-value ctl 'cap))
+	;; So, this hard coding is used.
+	(when (and (string-match "^Multipart/" ctr)
+		   (string-match "boundary=\"?\\([^\"\n\t;]+\\)\"?" ctr))
+	  (setq boundary (mew-match-string 1 ctr))
 	  (setq boundary (concat "^--" (regexp-quote boundary)))
 	  (catch 'loop
 	    (while (< i I) 
@@ -576,12 +530,11 @@ to messages and puts them to your mailbox, configure as follows:
 	    (unless ctr ;; not ""
 	      (setq textp t)
 	      (setq cs mew-cs-autoconv)
-	      (throw 'break nil))
-	    (setq ctl (mew-param-decode ctr))
-	    (setq ct (mew-syntax-get-value ctl 'cap))))
-	(when (string= ct mew-ct-txt)
-	  (setq charset (mew-syntax-get-param ctl "charset"))
-	  (if (and cte (mew-case-equal cte mew-b64)) (throw 'break nil))
+	      (throw 'break nil))))
+	(if (and cte (mew-case-equal cte mew-b64)) (throw 'break nil))
+	(when (string-match "^Text/Plain" ctr)
+	  (when (string-match "charset=\"?\\([^\"\n\t;]+\\)\"?" ctr)
+	    (setq charset (mew-match-string 1 ctr))) 
 	  ;; xxx quoted-printable. not enough DB in mew-mule3.el.
 	  (setq textp t)
 	  (setq cs (mew-charset-to-cs charset))
@@ -612,9 +565,7 @@ to messages and puts them to your mailbox, configure as follows:
   (if (and (integerp mew-summary-scan-width)
 	   (> mew-summary-scan-width 40)) ;; xxx
       mew-summary-scan-width
-    (if (< (window-width) 80)
-	80
-      (window-width))))
+    (max mew-window-magic (window-width))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -634,12 +585,17 @@ to messages and puts them to your mailbox, configure as follows:
       (if truncated
 	  (setq fields (concat uid " 0" siz)) ;; e.g. 0500 == truncated
 	(setq fields (concat uid " " siz)))
+      (save-excursion
+	(mew-header-delete-lines (list mew-x-mew-uidl:)))
       (mew-header-insert mew-x-mew-uidl: fields 'no-fold))))
 
 (defun mew-scan-message-truncatedp ()
   (let ((siz (mew-scan-uid-size (MEW-UID))))
     (and siz (string-match "^0" siz))))
 
+(defun mew-scan-message-invalidp ()
+  (and (MEW-NUM) (string-match "^0" (MEW-NUM))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Scanning a folder
@@ -697,7 +653,7 @@ non-nil, only headers of messages are cacched. If executed with
 	      ((mew-folder-popp fld)
 	       (if (mew-pop-header-only case)
 		   (setq get-body (not get-body)))
-	       (mew-pop-retrieve  case directive bnm range get-body))
+	       (mew-pop-retrieve case directive bnm range get-body))
 	      ((mew-folder-imapp fld)
 	       (if (mew-imap-header-only case)
 		   (setq get-body (not get-body)))
@@ -770,29 +726,17 @@ non-nil, only headers of messages are cacched. If executed with
 
 (defun mew-summary-folder-cache-load ()
   (let* ((folder (mew-summary-folder-name 'ext))
-	 (cache (mew-expand-folder folder mew-summary-cache-file))
-	 fld)
+	 (cache (mew-expand-folder folder mew-summary-cache-file)))
     (when (and (file-readable-p cache)
 	       (mew-summary-folder-cache-newp))
       (mew-elet
        (mew-erase-buffer)
        (mew-frwlet
 	mew-cs-m17n mew-cs-dummy
-	(insert-file-contents cache))
+	(mew-insert-file-contents cache))
        (mew-sinfo-set-cache-time (mew-file-get-time cache))
        (if (= (point-max) 1)
 	   (setq mew-summary-buffer-raw nil)
-	 (save-excursion
-	   (goto-char (point-min))
-	   (setq fld (mew-summary-folder-name 'ext))
-	   (unless (equal (mew-summary-folder-name) fld)
-	     (message "You may rename this folder. Fixing the format...")
-	     (while (re-search-forward mew-regex-sumsyn-short nil t)
-	       (goto-char (match-beginning 1))
-	       (delete-region (point) (match-end 1))
-	       (insert fld)
-	       (forward-line))
-	     (message "You may rename this folder. Fixing the format...done")))
 	 (setq mew-summary-buffer-raw t))
        (mew-mark-undo-mark mew-mark-refile 'no-msg)
        (mew-summary-set-count-line)
