@@ -207,7 +207,7 @@
       (when result
 	(delete-region (point-min) (point-max))
 	(mew-flet 
-	 (insert-file-contents decrypted-file)
+	 (mew-insert-file-contents decrypted-file)
 	 (put-text-property (point-min) (point-max) 'mew-noncontents nil)
 	 ;; because of RICH functionality of RFC1847... Gee dirty!
 	 (mew-decode-crlf-magic)))
