@@ -24,7 +24,7 @@
 	   (insert "\n"))
        (mew-message-clear-end-of)
        (when (or mew-end-of-message-string mew-end-of-part-string)
-	 (move-overlay (mew-minfo-get-overlay) (point-max) (point-max))
+	 (move-overlay (mew-minfo-get-eom) (point-max) (point-max))
 	 (if (mew-decode-syntax-p)
 	     (if (mew-summary-end-of-message-p)
 		 (mew-message-set-end-of-message)
@@ -32,8 +32,8 @@
 	   (mew-message-set-end-of-message)))))))
 
 (defsubst mew-message-clear-end-of ()
-  (unless (overlayp (mew-minfo-get-overlay))
-    (mew-minfo-set-overlay (mew-overlay-make (point-max) (point-max))))
+  (unless (overlayp (mew-minfo-get-eom))
+    (mew-minfo-set-eom (mew-overlay-make (point-max) (point-max))))
   (mew-message-set-end-of-nil))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -508,8 +508,8 @@ If called with '\\[universal-argument]', it stays writable."
      (mew-frwlet
       (if (mew-folder-draftp fld) mew-cs-m17n mew-cs-autoconv) mew-cs-dummy
       (if arg
-	  (find-file file)
-	(view-file file))))))
+	  (switch-to-buffer (mew-find-file-noselect2 file))
+	(view-buffer (mew-find-file-noselect2 file)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
