@@ -295,9 +295,10 @@
   (let (here)
     (save-excursion
       (goto-char (point-min))
-      (and (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
-	   (re-search-forward (mew-regex-jmp-part part) nil t)
-	   (setq here (point))))
+      (when (and (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+		 (re-search-forward (mew-regex-jmp-part part) nil t))
+	(beginning-of-line)
+	(setq here (point))))
     (and here (goto-char here))))
 
 (defun mew-summary-goto-message ()
@@ -319,7 +320,7 @@
       (setq str (toolbar-button-help-string (event-toolbar-button event)))
       ;; last-pressed-toolbar-button cannot be used.
       (if (string-match mew-regex-part-icon str)
-	  (setq ret (mew-sumsyn-part))))
+	  (setq ret (mew-match-string 1 str))))
     (or ret
 	(if (or (mew-in-attach-p)
 		(mew-in-decode-syntax-p))
@@ -361,16 +362,17 @@
   "Display the folder name of this message and the message number.
 If this is a cache message, both the unique id and the size are also shown."
   (interactive "P")
-  (when (mew-sumsyn-match mew-regex-sumsyn-long)
-    (if arg
-	(message "my-id=%s par-id=%s"
-		 (mew-sumsyn-my-id)
-		 (mew-sumsyn-parent-id))
-      (message "%s/%s size=%s uid=%s"
-	       (mew-sumsyn-folder-name)
-	       (mew-sumsyn-message-number)
-	       (mew-sumsyn-message-size)
-	       (mew-sumsyn-message-uid)))))
+  (mew-summary-msg
+   (when (mew-sumsyn-match mew-regex-sumsyn-long)
+     (if arg
+	 (message "my-id=%s par-id=%s"
+		  (mew-sumsyn-my-id)
+		  (mew-sumsyn-parent-id))
+       (message "%s/%s size=%s uid=%s"
+		(mew-sumsyn-folder-name)
+		(mew-sumsyn-message-number)
+		(mew-sumsyn-message-size)
+		(mew-sumsyn-message-uid))))))
 
 (defsubst mew-sumsyn-match (regex)
   (save-excursion
@@ -380,7 +382,15 @@ If this is a cache message, both the unique id and the size are also shown."
       (when (search-backward "\r" start t)
 	(looking-at regex)))))
 
-(defsubst mew-sumsyn-folder-name    () (mew-match-string 1))
+(defsubst mew-sumsyn-folder-name ()
+  (let ((fn (mew-match-string 1))
+	(bn (buffer-name)))
+    (cond
+     ((not (string= fn "")) fn)
+     ((mew-thread-p)
+      (mew-thread-to-folder bn))
+     (t bn))))
+
 (defsubst mew-sumsyn-message-number () (mew-match-string 2))
 (defsubst mew-sumsyn-my-id          () (mew-match-string 3))
 (defsubst mew-sumsyn-parent-id      () (mew-match-string 4))
@@ -743,7 +753,7 @@ If this is a cache message, both the unique id and the size are also shown."
 			  (mew-join "," decrypters-list)))
 	 (cd-or-dec cd)
 	 (privacy (mew-syntax-get-privacy syntax))
-	 (space " ") (LT (- (window-width) 2))
+	 (space " ") (LT (- (min mew-window-magic (window-width)) 2))
 	 (ln (length number))
 	 (lm 4) (lt 27)
 	 (ld (* (/ (- LT lm lt) 5) 3))
@@ -1008,16 +1018,17 @@ If this is a cache message, both the unique id and the size are also shown."
        (mew-rear-nonsticky (1- (point)) (point)))
      pos)))
 
-(defsubst mew-header-clear ()
+(defsubst mew-header-clear (&optional keep-read-only)
   ;; the cursor moves to the end of the header (with some exceptions)
   (mew-elet
    (mew-header-goto-end) ;; do not use mew-header-end
    (let ((pos (point)))
      (forward-line)
-     ;; If the body contains the read-only property, mew-header-p
+     ;; If the body contains the read-only property, mew-in-header-p()
      ;; makes a mistake. So, remove the read-only property from
      ;; the entire buffer.
-     (put-text-property (point) (point-max) 'read-only nil)
+     (unless keep-read-only
+       (put-text-property (point) (point-max) 'read-only nil))
      (delete-region pos (point)))))
 
 ;;
