@@ -178,40 +178,53 @@ threads for messages in the region.
 If called in Virtual mode, switch back to the corresponding Summary
 mode and move to the current message."
   (interactive "P")
-  (mew-summary-goto-message)
-  (mew-decode-syntax-delete)
-  (let* ((msg (mew-summary-message-number))
-	 (disp (mew-sinfo-get-disp-msg))
-	 (folder (mew-summary-folder-name 'ext)) ;; xxx
-	 fld vfolder)
-    (cond
-     ((mew-virtual-p)
-      (if (not (mew-thread-p))
-	  (message "No physical folder")
-	(setq fld (mew-thread-to-folder folder))
-	(if (not (and fld (get-buffer fld)))
-	    (message "No physical folder")
-	  (mew-summary-visit-folder fld nil 'no-ls)
-	  (mew-summary-toggle-disp-msg (if disp 'on 'off))
-	  (if (not msg)
-	      (goto-char (point-max))
-	    (mew-summary-move-and-display msg)))))
-     ((and (setq vfolder (mew-folder-to-thread folder))
-	   (get-buffer vfolder)
-	   (mew-virtual-thread-p vfolder)
-	   (mew-thread-cache-valid-p vfolder))
-      (mew-summary-visit-folder vfolder)
-      (mew-summary-toggle-disp-msg (if disp 'on 'off))
-      (when msg
-	(mew-summary-move-and-display msg)
-	(mew-thread-move-cursor)))
-     (arg
+  (if arg
       (let ((begend (mew-summary-get-region)))
-	(mew-summary-thread-region (car begend) (cdr begend))))
-     (t
-      (mew-summary-thread-region (point-min) (point-max) nil msg)))))
+	(mew-summary-thread-region (car begend) (cdr begend)))
+    (mew-summary-goto-message)
+    (mew-decode-syntax-delete)
+    (let* ((msg (mew-summary-message-number))
+	   (disp (mew-sinfo-get-disp-msg))
+	   (folder (mew-summary-folder-name 'ext)) ;; xxx
+	   fld vfolder)
+      (cond
+       ((mew-virtual-p)
+	(if (not (mew-thread-p))
+	    (message "No physical folder")
+	  (setq fld (mew-thread-to-folder folder))
+	  (if (not (and fld (get-buffer fld)))
+	      (message "No physical folder")
+	    (mew-summary-visit-folder fld nil 'no-ls)
+	    (mew-summary-toggle-disp-msg (if disp 'on 'off))
+	    (if (not msg)
+		(goto-char (point-max))
+	      (mew-summary-move-and-display msg)))))
+       ((and (setq vfolder (mew-folder-to-thread folder))
+	     (get-buffer vfolder)
+	     (mew-virtual-thread-p vfolder)
+	     (mew-thread-cache-valid-p vfolder))
+	(mew-summary-visit-folder vfolder)
+	(mew-summary-toggle-disp-msg (if disp 'on 'off))
+	(when msg
+	  (mew-summary-move-and-display msg)
+	  (mew-thread-move-cursor)))
+       (t
+	(mew-summary-thread-region (point-min) (point-max) nil msg))))))
 
-(defun mew-summary-thread-region (beg end &optional mark disp-msg)
+(defun mew-summary-regexp-make-thread ()
+  "Make threads for messages matched to a specified regular expression."
+  (interactive)
+  (mew-decode-syntax-delete)
+  (let ((regex "") iter)
+    (while (string= regex "")
+      (setq regex (read-string "Regexp: ")))
+    (setq iter
+          (function
+           (lambda ()
+             (re-search-forward regex nil t))))
+    (mew-summary-thread-region (point-min) (point-max) 'mark nil iter)))
+
+(defun mew-summary-thread-region (beg end &optional mark disp-msg iter)
   "Make threads for messages in a region.  If you want to know how
 threads are created, see 'mew-use-complete-thread'."
   (interactive "r")
@@ -245,6 +258,8 @@ threads are created, see 'mew-use-complete-thread'."
 	       (setq func (lambda ()
 			    (re-search-forward mew-regex-msg-review nil t)))
 	     (setq func (lambda () (not (eobp)))))
+           (if iter
+               (setq func iter))
 	   (setq tm1 (current-time))
 	   (while (funcall func)
 	     (beginning-of-line)
@@ -532,11 +547,8 @@ with the current folder as a candidate in addition to guessed folders."
 	(beginning-of-line)
 	(setq bottom (point))
 	(mew-summary-mark-in-physical-alist alist mark func))
-      ;; for C-x C-x
-      (let ((zmacs-regions nil)
-	    ;; To treat this thread as if it were a single message.
-	    (mew-summary-down-function (lambda () (goto-char bottom))))
-	(push-mark (point) t t)
+      (mew-push-mark)
+      (let ((mew-summary-down-function (lambda () (goto-char bottom))))
 	(mew-summary-display-after mew-summary-mark-direction))
       alist))))
 
@@ -863,8 +875,8 @@ a top node, move onto the top of the previous thread."
 	(mew-thread-change-parent-id id)
 	(mew-thread-save-xref fld msg id)
 	(if (mew-virtual-p)
-	    (mew-summary-change-parent-id fld msg id)))
-      (sit-for 0.1))))
+	    (mew-summary-change-parent-id fld msg id))
+	(message "Glued")))))
 
 (defun mew-thread-glue-it ()
   (let ((column (mew-vinfo-get-column))
@@ -973,7 +985,7 @@ a top node, move onto the top of the previous thread."
 (defun mew-thread-save-xref (fld msg id)
   (with-temp-buffer
     (let ((file (mew-expand-folder fld msg)))
-      (insert-file-contents file)
+      (mew-insert-file-contents2 file)
       (mew-header-delete-lines (list mew-x-mew-ref:))
       (goto-char (point-min))
       (mew-header-insert mew-x-mew-ref: id)
