@@ -174,11 +174,26 @@ really deleted."
   :group 'mew-folder
   :type 'string)
 
-(defcustom mew-imap-friend-folder "%inbox.from" ;; cyrus
+(defcustom mew-imap-friend-folder "%from"
   "*The folder where personal messages are refiled for IMAP."
   :group 'mew-folder
   :type 'string)
 
+(defcustom mew-imap-spam-field nil
+  "*The field name of anti spam information to be used for IMAP spam filter. 
+If both 'mew-imap-spam-field' and 'mew-imap-spam-word' are defined, 
+messages whose the field contains the word are automatically
+removed or refiled to 'mew-imap-trash-folder' (if defined) 
+while scanning %inbox."
+  :group 'mew-imap
+  :type 'string)
+
+(defcustom mew-imap-spam-word nil
+  "*The word of anti spam information to be used for IMAP spam filter. 
+See 'mew-imap-spam-field'."
+  :group 'mew-imap
+  :type 'string)
+
 ;; must be here
 (defvar mew-basic-folders
   `(,mew-inbox-folder ,mew-draft-folder ,mew-trash-folder
@@ -293,12 +308,12 @@ Note that this hook is called after composition of the message."
   :type 'hook)
 
 (defcustom mew-smtp-flush-hook nil
-  "*Hook called after the queue is flushed."
+  "*Hook called before SMTP runs if messages exist in +queue."
   :group 'mew-draft
   :type 'hook)
 
 (defcustom mew-nntp2-flush-hook nil
-  "*Hook called after the postq is flushed."
+  "*Hook called before NNTP runs if messages exist in +postq"
   :group 'mew-draft
   :type 'hook)
 
@@ -748,6 +763,8 @@ If non-nil, only header is retrieved."
 
 (defvar mew-imap-timeout-time 4)
 
+(defvar mew-imap-cache-refile-online t)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; NNTP
@@ -1100,13 +1117,13 @@ quantization by `mew-summary-form-size-unit'."
   :group 'mew-summary
   :type 'boolean)
 
-(defvar mew-spam: "X-Spam-Flag:"
+(defvar mew-spam: "X-Bogosity:"
   "*The field name which your spam filter inserts. 
+This variable MUST be set in \".emacs\".
 E.g.
 	\"X-Spam-Flag:\" for 'spamassassin'
 	\"X-Bogosity:\" for 'bogofilter'")
 
-
 (defcustom mew-scan-fields
   `("Folder:" "Filename:" 
     ,mew-subj: ,mew-date: ,mew-from: ,mew-to: ,mew-cc:
@@ -1134,6 +1151,8 @@ Functions called MEW-FOO will be defined according to this variable.")
 (defvar mew-summary-form-func-prefix "mew-summary-form-"
   "The prefix for symbol used in 'mew-summary-form'.")
 
+(defvar mew-scan-decode-fields `(,mew-subj: ,mew-from: ,mew-to:))
+
 (defcustom mew-thread-column 28
   "A position where thread indentation is inserted
 for indentation of thread. This valuable is for backward compatibility
@@ -1215,7 +1234,7 @@ to this file."
   :type 'integer)
 
 (defcustom mew-highlight-body-max-size 10000
-  "A limit for highlighting a body."
+  "A limit for highlighting a body. '0' means No-Limitation"
   :group 'mew-highlight
   :type 'integer)
 
@@ -1316,6 +1335,23 @@ is appropriate.
   :group 'mew-draft
   :type '(choice (const nil) (const t) (repeat string)))
 
+(defcustom mew-ask-folder-after-join nil
+  "*If *non-nil*, Mew asks to move to the folder where the join
+message is stored."
+  :group 'mew-summary
+  :type 'boolean)
+
+(defcustom mew-ask-folder-after-burst nil
+  "*If *non-nil*, Mew asks to move to the folder where the bursted
+messages are stored."
+  :group 'mew-summary
+  :type 'boolean)
+
+(defcustom mew-ask-pipe nil
+  "*If *non-nil*, Mew confirms when \"\\<mew-summary-mode-map>\\[mew-summary-pipe-message]\" is used."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Use
@@ -1454,11 +1490,21 @@ Typing '%' automatically inserts 'mew-case-input' before '%'."
   :type 'boolean)
 
 (defcustom mew-use-spc-padding t
-  "*If non-nil SPCs are padded from the end of lines of Summary mode 
+  "*If non-nil, SPCs are padded from the end of lines of Summary mode 
 to the right side."
   :group 'mew-summary
   :type 'boolean)
 
+(defcustom mew-use-push-mark nil
+  "*If non-nil, \\<mew-summary-mode-map>\\[mew-summary-jump-top] and \\<mew-summary-mode-map>\\[mew-summary-jump-bottom] push the mark."
+  :group 'mew-summary
+  :type 'boolean)
+
+(defcustom mew-use-retrieve t
+  "*If non-nil, '\\<mew-summary-mode-map>\\[mew-summary-retrieve]' can be used."
+  :group 'mew-summary
+  :type 'boolean)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Appearance
@@ -1493,6 +1539,8 @@ Otherwise, not displayed."
   :group 'mew-summary
   :type 'string)
 
+(defvar mew-window-magic 80)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Summary mode
@@ -1609,6 +1657,10 @@ hidden before visible fields (and after invisible fields)."
   :group 'mew-message
   :type 'boolean)
 
+(defvar mew-use-header-veil t)
+(defvar mew-header-veil-string "...")
+(defvar mew-header-veil-count 4)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Draft mode
@@ -1766,6 +1818,11 @@ Otherwise, the cursor is after To:."
   :group 'mew-cite
   :type 'symbol)
 
+(defcustom mew-cite-ignore-region nil
+  "*If *non-nil*, the region of a draft is ignored when citing."
+  :group 'mew-cite
+  :type 'boolean)
+
 (defcustom mew-cite-ignore-mouse-region t
   "*If *non-nil*, a region specified by mouse is ignored when citing."
   :group 'mew-cite
@@ -2558,16 +2615,38 @@ your public key as well as receivers' one."
 ;;; PGP Public key fetch
 ;;;
 
-(defvar mew-pgp-keyserver-url-template
-  "http://pgp.mit.edu:11371/pks/lookup?op=get&exact=on&search=%s"
-;; "http://pgp.nic.ad.jp:11371/pks/lookup?op=get&exact=on&search=%s"
-  )
-
 (defvar mew-x-pgp-key-list
   '("x-pgp-key:" "x-pgp-key-url:" "x-pgp-public-key:" "x-pgp-public-key-url:"
     "x-pgp5-key:" "x-pgp5-key-url:" "x-pgp5-public-key:"
     "x-pgp5-public-key-url:" "x-public-key:"))
    
+(defvar mew-pgp-pks-port "11371")
+(defvar mew-pgp-pks-lang "/pks/lookup?op=get&exact=on&search=")
+
+;; Tue Dec 16 16:28:11 JST 2003
+;; http://pgp.zdv.uni-mainz.de/keyserver/bigbrother/
+
+(defvar mew-pgp-pks-servers
+  '(("openpksd.org" t)
+    ("blackhole.pca.dfn.de" nil)
+    ("pgp.ael.be" nil)
+    ("pgp.dtype.org" nil)
+    ("pgp.mit.edu" nil)
+    ("pgp.nic.ad.jp" nil)
+    ("pgp.rediris.es" nil)
+    ("pgp.uni-mainz.de" nil)
+    ("pgp.uni-paderborn.de" nil)
+    ("wwwkeys.1.us.pgp.net" nil)
+    ("wwwkeys.2.us.pgp.net" nil)
+    ("wwwkeys.3.us.pgp.net" nil)
+    ("wwwkeys.ch.pgp.net" nil)
+    ("wwwkeys.cz.pgp.net" nil)
+    ("wwwkeys.es.pgp.net" nil)
+    ("wwwkeys.uk.pgp.net" nil)))
+
+(defun mew-pgp-pks-http-port-p (server alist)
+  (nth 1 (assoc server alist)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; SSH
@@ -2594,10 +2673,10 @@ your public key as well as receivers' one."
   :type 'string)
 
 (defvar mew-prog-pgpkey      "wget")
-(defvar mew-prog-pgpkey-arg  '("-q" "-O" "-"))
+(defvar mew-prog-pgpkey-args '("-q" "-O" "-"))
 
-(defvar mew-prog-ssh          "ssh")
-(defvar mew-prog-ssh-arg      nil)
+(defvar mew-ssh-prog          "ssh")
+(defvar mew-ssh-prog-args     nil)
 
 (defvar mew-prog-uncompface  "uncompface")
 (defvar mew-prog-icontopbm   "icontopbm")
@@ -2638,6 +2717,23 @@ w3 is used."
   :group 'mew-summary
   :type '(choice file (const nil)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; spam and ham
+;;;
+
+(defvar mew-spam-prog "bogofilter"
+  "*A command which learns spams.")
+
+(defvar mew-spam-prog-args '("-s" "-N" "-v")
+  "*A list of command arguments which learns spams.")
+
+(defvar mew-ham-prog "bogofilter"
+  "*A command which learns hams.")
+
+(defvar mew-ham-prog-args '("-n" "-S" "-v")
+  "*A list of command arguments which learns hams.")
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Mode names
@@ -2674,7 +2770,7 @@ Good example:
 Bad example:
 	Subject: Raw non-ASCII text.
 
-(3) Decode illegal MIME parameters, which violates RFC 2231.
+(3) Decode invalid MIME parameters, which violates RFC 2231.
 (i)  Raw text.
 (ii) Encorded-word defined in RFC 2047.
 
@@ -2686,7 +2782,7 @@ Bad example
 	Content-Disposition: attachment;
 	 filename=\"=?iso-2022-jp?B?GyRCJEYkOSRIGyhC?=\"
 
-(4) Decode illegal non-ASCII MIME body, whose charset is US-ASCII or
+(4) Decode invalid non-ASCII MIME body, whose charset is US-ASCII or
 not present.
 
 Good example:
@@ -2699,7 +2795,7 @@ Bad example:
 
 	ISO-2022-JP text
 
-(5) Decode illegal non-ASCII text, whose charset is UTF-8 or not present.
+(5) Decode invalid non-ASCII text, whose charset is UTF-8 or not present.
 
 Good example:
 	Content-Type: text/plain; charset=ISO-8859-15
@@ -2715,7 +2811,7 @@ Bad example:
   :type 'boolean)
 
 (defcustom mew-use-name-parameter t
-  "*If non-nil, the illegal NAME parameter of Content-Type: is checked
+  "*If non-nil, the invalid NAME parameter of Content-Type: is checked
 as a file name in addition to the legal FILENAME parameter of
 Content-Disposition:."
   :group 'mew-message
