@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: May 22, 2003
+;; Revised: Dec 25, 2003
 
 ;;; Commentary:
 
@@ -19,7 +19,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "4.0.53"
+(defconst mew-version-number "4.0.62"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -245,7 +245,8 @@ If executed with '3', information about IMAP folders is updated."
     (mew-theme-setup)
     (mew-biff-setup)
     (mew-ct-setup)))
-  (message "Updating status...done"))
+  (unless (or (eq arg 2) (eq arg 3))
+    (message "Updating status...done")))
 
 (defvar mew-mime-content-type-list nil
   "Candidate of Content-Type: when CT: is changed in draft buffer.")
@@ -402,7 +403,8 @@ the lower window is not zero, switch to the buffer."
     (unless (zerop msg-height)
       (split-window nil sum-height)
       (other-window 1)
-      (switch-to-buffer msgbuf 'norecord))))
+      (switch-to-buffer msgbuf 'norecord)
+      (mew-set-buffer-cs mew-cs-m17n))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -693,6 +695,7 @@ Mew remain, so you can resume with buffer operations."
 (require 'mew-nntp)
 (require 'mew-nntp2)
 (require 'mew-imap)
+(require 'mew-imap2)
 (require 'mew-config)
 (require 'mew-auth)
 
